Package: BREADR
Title: Estimates Degrees of Relatedness (Up to the Second Degree) for
        Extreme Low-Coverage Data
Version: 1.1.0
Authors@R: 
    c(
    person(
      given = "Jono", 
      family = "Tuke", 
      email = "simon.tuke@adelaide.edu.au", 
      role = c("aut", "cre"),
      comment = c(ORCID = "0000-0002-1688-8951")
    ), 
    person(
      given = "Adam B.", 
      family = "Rohrlach",
      email = "adam.rohrlach@adelaide.edu.au", 
      role = "aut",
      comment = c(ORCID = "0000-0002-4204-5018")
    ), 
    person(
      given = "Wolfgang", 
      family = "Haak",
      email = "wolfgang_haak@eva.mpg.de", 
      role = "aut",
      comment = c(ORCID = "0000-0003-2475-2007")
    ), 
    person(
      given = "Divyaratan",
      family = "Popli",
      email = "divyaratan_popli@eva.mpg.de",
      role = "aut", 
      comment = c(ORCID = "0000-0002-0305-6427")
    )
    )
Description: The goal of the package is to provide an easy-to-use method 
  for estimating degrees of relatedness (up to the second degree) 
  for extreme low-coverage data. The package also allows users to 
  quantify and visualise the level of confidence in the estimated 
  degrees of relatedness.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://github.com/jonotuke/BREADR,
        https://jonotuke.github.io/BREADR/
BugReports: https://github.com/jonotuke/BREADR/issues
Suggests: spelling, knitr, rmarkdown, testthat (>= 3.0.0), Matrix, fs
Config/testthat/edition: 3
Imports: data.table, dplyr, forcats, ggplot2, ggpubr, ggstatsplot,
        grDevices, magrittr, MASS, matrixStats, purrr, readr, stringr,
        tibble, tidyr
Depends: R (>= 4.3)
LazyData: true
Language: en-US
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-16 06:32:17 UTC; a1078881
Author: Jono Tuke [aut, cre] (ORCID: <https://orcid.org/0000-0002-1688-8951>),
  Adam B. Rohrlach [aut] (ORCID: <https://orcid.org/0000-0002-4204-5018>),
  Wolfgang Haak [aut] (ORCID: <https://orcid.org/0000-0003-2475-2007>),
  Divyaratan Popli [aut] (ORCID: <https://orcid.org/0000-0002-0305-6427>)
Maintainer: Jono Tuke <simon.tuke@adelaide.edu.au>
Repository: CRAN
Date/Publication: 2026-04-16 07:02:46 UTC
Built: R 4.5.2; ; 2026-04-20 23:49:17 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(callRelatedness)
export(get_column_new)
export(plotDOUGH)
export(plotLOAF)
export(plotSLICE)
export(priorSensitivity)
export(processEigenstrat)
export(processEigenstrat_old)
export(read_ind)
export(read_snp)
export(saveSLICES)
export(sim_geno)
export(split_line)
export(test_degree)
import(data.table)
importFrom(magrittr,"%>%")

<!-- README.md is generated from README.Rmd. Please edit that file -->

# BREADR

<!-- badges: start -->

[![R-CMD-check](https://github.com/jonotuke/BREADR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/jonotuke/BREADR/actions/workflows/R-CMD-check.yaml)

[![DOI](https://joss.theoj.org/papers/10.21105/joss.07916/status.svg)](https://doi.org/10.21105/joss.07916)
<!-- badges: end -->

The goal of BREADR is to provide an easy-to-use method for estimating
degrees of relatedness (up to the second degree) for extremely
low-coverage data. BREADR also allows users to quantify and visualise
the level of confidence in the estimated degrees of relatedness.

The method requires Eigenstrat files (an ind, geno and snp file) to
begin, allowing the user to account for DNA deamination when genotyping
their data.

Further information can be found at the [BREADR
website](https://jonotuke.github.io/BREADR/).

## Installation

To install, you can use the usual

``` r
install.packages("BREADR")
```

You can install the development version of BREADR from
[GitHub](https://github.com/) with:

``` r
# install.packages("remotes")
remotes::install_github("jonotuke/BREADR")
```

Also to ensure that BREADR install correctly, we suggest installing the
following packages

- Matrix,
- data.table,
- dplyr,
- forcats,
- ggplot2,
- ggpubr,
- grDevices,
- magrittr,
- MASS,
- matrixStats,
- purrr,
- readr,
- stringr and
- tibble.

## Contributing to BREADR

Please note that `BREADR` is work in progress! If you are interested in
the project and want to know more please feel free to contact Jono Tuke
(<simon.tuke@adelaide.edu.au>). If you find a bug or would like to see
new or improved features, please open an issue on the
[GitHub](https://github.com/jonotuke/BREADR) repository.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(BREADR)

## -----------------------------------------------------------------------------
ind_path <- "path/to/eigenstrat/indfile"
snp_path <- "path/to/eigenstrat/snpfile"
geno_path <- "path/to/eigenstrat/genofile"

## ----eval=FALSE---------------------------------------------------------------
# counts_example <- processEigenstrat(
#   indfile = ind_path,
#   snpfile = snp_path,
#   genofile = geno_path
# )

## ----eval=FALSE---------------------------------------------------------------
# counts_example <- processEigenstrat(
#   indfile = ind_path,
#   snpfile = snp_path,
#   genofile = geno_path,
#   outfile = "path_to_save_tsv"
# )

## ----example, eval = FALSE----------------------------------------------------
# library(BREADR)
# counts_example

## ----echo = FALSE-------------------------------------------------------------
counts_example

## ----eval = FALSE-------------------------------------------------------------
# relatedness_example <- callRelatedness(counts_example)
# relatedness_example

## ----echo = FALSE-------------------------------------------------------------
relatedness_example <- callRelatedness(counts_example)
relatedness_example

## ----fig.dim=c(7.2,5), warning=FALSE, message=FALSE, fig.cap = '*__Figure 1__: plotDOUGH results for the example data set, with the Between Groups population kept.*'----
indfile1 <- fs::path_package(package = "BREADR", "extdata/example.ind.txt")
plotDOUGH(
  counts_example,
  indfile = indfile1,
  nsnps_cutoff = 500
)

## ----echo=FALSE, fig.dim=c(7.2,5), warning=FALSE, message=FALSE, fig.cap = '*__Figure 2__: plotDOUGH results for the example data set where the PMR values have been artifically modified to yield different baseline (median) PMR values.*'----
indfile2 <- fs::path_package(package = "BREADR", "extdata/example2.ind.txt")
counts_example %>%
  dplyr::mutate(
    mismatch = dplyr::case_when(
      stringr::str_count(pair, 'Ind[123]') == 2 ~ floor(0.22 * nsnps),
      stringr::str_count(pair, 'Ind[456]') == 2 ~ floor(0.16 * nsnps),
      T ~ floor(0.19 * nsnps)
    ) +
      sample(10:20, dplyr::n(), replace = TRUE),
    pmr = mismatch / nsnps
  ) %>%
  plotDOUGH(
    indfile = indfile2,
    nsnps_cutoff = 500
  )

## ----warning=FALSE, message=FALSE, fig.dim=c(7.2,5), fig.cap = '*__Figure 3__: Pairwise diagnostic plot for relatedness call for Ind1 and Ind 2 called using the number option for row. (A) the distribution of PMR values given the number of overlapping SNPs (n=1518) for each degree of relatedness (filled colour), and the observed PMR with 95% confidence intervals. (B) relative posterior probabilities for all four possible assignments of degrees of relatedness. Note we added panel labels A and B using the labels parameter.*'----
plotLOAF(relatedness_example)

## ----fig.dim=c(7.2,5), warning=FALSE, fig.cap = '*__Figure 4__: Pairwise diagnostic plot for relatedness call for Ind1 and Ind 2 called using the text option for row. (A) the distribution of PMR values given the number of overlapping SNPs (n=1518) for each degree of relatedness (filled colour), and the observed PMR with 95% confidence intervals. (B) relative posterior probabilities for all four possible assignments of degrees of relatedness. Note we added panel labels A and B using the labels parameter.*'----
plotSLICE(relatedness_example, row = 1, labels = c('A', 'B'))

## ----fig.dim=c(7.2,5), warning=FALSE, fig.cap = '*__Figure 5__: Pairwise diagnostic plot for relatedness call for Ind1 and Ind 2 called using the text option for row. (A) the distribution of PMR values given the number of overlapping SNPs (n=1518) for each degree of relatedness (filled colour), and the observed PMR with 95% confidence intervals. (B) relative posterior probabilities for all four possible assignments of degrees of relatedness. Note we added panel labels A and B using the labels parameter.*'----
plotSLICE(relatedness_example, row = "Ind1 - Ind2", labels = c('A', 'B'))

## -----------------------------------------------------------------------------
test_degree(relatedness_example, row = 1, degree = 3, verbose = TRUE)

## ----fig.dim=c(7.2,5), warning=FALSE, message=FALSE, fig.cap = '*__Figure 6__: sensitivity analysis for the relationship between Ind1 and Ind2. The prior probabilities for all degrees of relatedness are tested at values between 0.05 and 0.5. Bars indicate the proportion of times a degree of relatedness was the maximum posterior relationship. Here we see that higher prior probabilities for Second-Degree and Unrelated yield highest posterior probability assignments for those degrees of relatedness, respectively. Hence, we must consider these prior probabilities carefully when assigning this relationship.*'----
BREADR::priorSensitivity(
  in_BREADR = relatedness_example,
  row = "Ind1 - Ind2",
  degrees = c(3, 4),
  grid_space = 0.05,
  maxPrior = 0.5
)

## ----fig.dim=c(7.2,3.5), warning=FALSE, message=FALSE,fig.cap = '*__Figure 7__: sensitivity analysis for the relationship between Ind3 and Ind4. The prior probabilities for all degrees of relatedness are tested at values between 0.05 and 0.5. Bars indicate the proportion of times a degree of relatedness was the maximum posterior relationship, and the dashed black line indicates 50%. This clearly shows that no matter the prior probability values for any of the degrees of relatedness, the highest posterior probability assignment is always Same/Twins.*'----
BREADR::priorSensitivity(
  in_BREADR = relatedness_example,
  row = "Ind3 - Ind4",
  degrees = c(1),
  grid_space = 0.025,
  maxPrior = 0.5
)

#' split line
#'
#' takes a line for a SNP file and splits into parts.
#' @param x line from SNP file
#'
#' @return tibble with 6 columns.
#' @export
#'
#' @examples
#' split_line("1_14.570829090394763     1        0.000000              14 A X")
#' split_line("rs3094315	1	0.0	752566	G	A")
split_line <- function(x){
  x <- stringr::str_trim(x)
  df <- stringr::str_split(x, " +|\t") %>% purrr::pluck(1)
  stopifnot(length(df) == 6)
  df <- tibble::tibble(
    snp = df[1],
    chr = as.numeric(df[2]),
    pos = as.numeric(df[3]),
    site = as.numeric(df[4]),
    anc = df[5],
    der = df[6]
    )
  return(df)
}
#' read_snp
#'
#' @param filename a SNP text file.
#'
#' @return tibble with column headings: snp (CHR), chr (DBL),
#' pos (DBL), site (DBL), anc (CHR), and der (CHR).
#' @export
#'
#' @examples
#' std_snpfile <- system.file("extdata", "example.snp.txt", package = "BREADR")
#' broken_snpfile <- system.file("extdata", "broken.snp.txt", package = "BREADR")
#' read_snp(std_snpfile)
#' read_snp(broken_snpfile)
read_snp <- function(filename){
    df <- data.table::fread(
      filename, header = FALSE,
      col.names=c("snp", "chr", "pos", "site", "anc", "der"),
      colClasses=list(character=c(1,2,5,6))
    )
    df <- tibble::as_tibble(df)
    return(df)
}
# pacman::p_load(tidyverse, targets)
# read_snp("inst/extdata/broken.snp.txt")
# read_snp("inst/extdata/example.snp.txt")

test_that("split_line works", {
  expect_error(split_line("1\t1"))
  expect_equal(as.character(split_line("rs3094315\t1\t0.0\t752566\tG\tA")[1,1]), "rs3094315")
  expect_equal(as.numeric(split_line("rs3094315\t1\t0.0\t752566\tG\tA")[1,2]), 1)
})

#' test_degree
#'
#' Test if a degree of relatedness is consistent with an observed PMR
#'
#' @param in_tibble a tibble that is the output of the callRelatedness() function.
#' @param row either the row number or pair name for which the posterior distribution is to be plotted.
#' @param degree the degree of relatedness to be tested.
#' @param verbose a logical (boolean) for whether all test output should be printed to screen.
#'
#' @return the associated p-value for the test
#' @export
#'
#' @examples
#' test_degree(relatedness_example, 1, 1)
test_degree <- function(in_tibble, row, degree, verbose = TRUE) {
  # Test that the in_tibble is of the correct form
  if (nrow(in_tibble) == 0) {
    stop('The input tibble is empty')
  }
  col_names_check <- c(
    'row',
    'pair',
    'relationship',
    'pmr',
    'sd',
    'mismatch',
    'nsnps',
    'ave_rel',
    'Same_Twins',
    'First_Degree',
    'Second_Degree',
    'Unrelated'
  )
  if ((!all(col_names_check %in% names(in_tibble)))) {
    stop(paste0(
      'Input tibble/data.frame must have 12 columns named: ',
      paste0(col_names_check, collapse = ', '),
      '.'
    ))
  }

  # check row is a numeric or char variable, and if char, find the row.
  if (is.character(row)) {
    rowtmp <- row
    row <- which(row == in_tibble$pair)
    if (length(row) == 0) {
      stop(paste0('There is no pair called: ', rowtmp))
    }
  } else if (is.numeric(row) & (row > nrow(in_tibble))) {
    stop(
      'row number requested was higher than the number of rows in input tibble!'
    )
  } else if (!is.numeric(row)) {
    stop('row must be a number or a pair name.')
  }

  if (!is.numeric(degree)) {
    stop('Degree must be numeric.')
  }
  # if(degree>10|degree<0|(degree%%1!=0)){ # Removed constraint that integer
  if (degree > 10 | degree < 0) {
    stop('Degree must be a value k such that 0<k<10.')
  }

  M <- in_tibble$ave_rel[row]
  pk <- (1 - 0.5^(degree + 1)) * M
  N <- in_tibble$nsnps[row]
  x <- in_tibble$mismatch[row]
  pair <- in_tibble$pair[row]
  # pval <- 2*stats::pbinom(x,N,pk,ifelse(x>pk*N,F,T))
  pval <- min(
    c(
      2 * stats::pbinom(x, N, pk, T),
      2 * stats::pbinom(x, N, pk, F)
    )
  )
  # CI <- x/N+c(-1,1)*1.96*sqrt(x/N*(1-x/N)/N)
  pvaltext <- ifelse(
    pval < 0.0001,
    sprintf('p-value          : %s\n', format(pval, scientific = T)),
    sprintf('p-value          : %.4f\n', pval)
  )

  degree_string <- dplyr::case_when(
    degree == 1 ~ "1st",
    degree == 2 ~ "2nd",
    degree == 3 ~ "3rd",
    T ~ paste0(degree, "th")
  )
  if (verbose) {
    cat(sprintf(
      'Testing H0       : "%s" are %s-degree relatives.\n',
      pair,
      degree_string
    ))
    cat(sprintf('Expected PMR     : %.4f\n', pk))
    cat(sprintf('Observed PMR     : %.4f\n', x / N))
    cat(sprintf('Estimated degree : %.4f\n', convertP(x / N, M)))
    cat(pvaltext)
    cat(ifelse(
      pval >= 0.05,
      'Decision         : Retain H0\n',
      'Decision         : Reject H0\n'
    ))
  }
  return(pval)
}

test_that("test_degree errors", {
  empty_tibble <- tibble::tribble(
    ~name, ~value
  )
  false_example <- relatedness_example %>% dplyr::rename(bob = pair)
  expect_error(test_degree(empty_tibble))
  expect_error(test_degree(false_example))
  expect_error(test_degree(relatedness_example, row = 100))
  expect_error(test_degree(relatedness_example, row = 1, degree = "bob"))
  expect_equal(test_degree(relatedness_example, row = 1, degree = 1), 2.667e-5,
               tolerance = 0.0001)
})

#' plotLOAF
#'
#' Plots all (sorted by increasing value) observed PMR values with
#' maximum posterior probability classifications represented by colour
#' and shape.
#' Options include a cut off for the minimum number of overlapping SNPs,
#' the max number of pairs to plot and x-axis font size.
#'
#' @param in_tibble a tibble that is the output of the callRelatedness() function.
#' @param nsnps_cutoff the minimum number of overlapping SNPs for which
#' pairs are removed from the plot. If NULL, default is 500.
#' @param N the number of (sorted by increasing PMR) pairs to plot.
#' Avoids plotting all pairs (many of which are unrelated).
#' @param fntsize the fontsize for the x-axis names.
#' @param verbose if TRUE, then information about the plotting process is sent
#' to the console
#'
#' @return a ggplot object
#' @export
#'
#' @examples
#' relatedness_example
#' plotLOAF(relatedness_example)
plotLOAF <- function(
  in_tibble,
  nsnps_cutoff = NULL,
  N = NULL,
  fntsize = 7,
  verbose = TRUE
) {
  # Test that the in_tibble is of the correct form
  if (nrow(in_tibble) == 0) {
    stop('The input tibble is empty')
  }
  col_names_check <- c(
    'row',
    'pair',
    'relationship',
    'pmr',
    'sd',
    'mismatch',
    'nsnps',
    'ave_rel',
    'Same_Twins',
    'First_Degree',
    'Second_Degree',
    'Unrelated'
  )
  if ((!all(col_names_check %in% names(in_tibble)))) {
    stop(paste0(
      'Input tibble/data.frame must have at least 12 columns named: ',
      paste0(col_names_check, collapse = ', '),
      '.'
    ))
  }

  # Test that the overlapping snp cut off is sensible.
  if (is.null(nsnps_cutoff)) {
    nsnps_cutoff <- 5e2
    if (verbose) {
      cat(sprintf(
        'No minimum number of overlapping SNPs given.\nUsing default minimum of 500.\n'
      ))
    }
  }
  if ((nsnps_cutoff <= 0)) {
    stop(
      'The cut off for the number of overlapping snps must be greater than 0.'
    )
  } else if (nsnps_cutoff >= max(in_tibble$nsnps, na.rm = T)) {
    stop(
      'The cut off for the number of overlapping snps cannot be greater than the maximum number of overlapping SNPs that were observed!'
    )
  }

  # test that the number of pairs to plot is sensible.
  if (is.null(N)) {
    N <- min(nrow(in_tibble), 50)
    if (verbose) {
      cat(sprintf(
        'No upper limit on number of pairs to plot given.\nPlotting first %i pairs.\n',
        N
      ))
    }
  } else if (N > sum(in_tibble$nsnps > nsnps_cutoff)) {
    N <- sum(in_tibble$nsnps > nsnps_cutoff)
    if (verbose) {
      cat(sprintf(
        'Number of pairs to plot was greater than the number of available pairs.\nPlotting first %i pairs.\n',
        N
      ))
    }
  }

  # Define class breaks
  class_breaks <- tibble::tibble(
    relationship = factor(
      c('Same_Twins', 'First_Degree', 'Second_Degree', 'Unrelated'),
      levels = c('Same_Twins', 'First_Degree', 'Second_Degree', 'Unrelated')
    ),
    meanPMR = ((1 - 0.5^c(1:3, Inf))) * in_tibble$ave_rel[1],
    col = ggcolorhue(4)
  )
  # Produce plot
  gg2 <- in_tibble %>%
    dplyr::arrange(pmr) %>%
    dplyr::mutate(
      relationship = factor(
        relationship,
        levels = c('Same_Twins', 'First_Degree', 'Second_Degree', 'Unrelated')
      )
    ) %>%
    dplyr::mutate(ymin = pmr - 2 * sd, ymax = pmr + 2 * sd) %>%
    dplyr::filter((nsnps >= nsnps_cutoff)) %>%
    dplyr::slice(1:N) %>%
    ggplot2::ggplot(ggplot2::aes(
      x = forcats::fct_reorder(pair, pmr),
      y = pmr
    )) +
    ggplot2::geom_hline(
      ggplot2::aes(yintercept = meanPMR, col = relationship),
      linetype = 'dashed',
      data = class_breaks,
      linewidth = 1
    ) +
    ggplot2::theme_bw() +
    ggplot2::geom_errorbar(ggplot2::aes(
      forcats::fct_reorder(pair, pmr),
      ymin = ymin,
      ymax = ymax
    )) +
    ggplot2::geom_point(
      ggplot2::aes(fill = relationship, shape = relationship),
      size = 2,
      show.legend = TRUE
    ) +
    ggplot2::ylab("PMR") +
    ggplot2::xlab(" ") +
    # scale_y_continuous(limits=c(0,0.25))+
    ggplot2::scale_fill_discrete(drop = FALSE, name = NULL) +
    ggplot2::scale_colour_discrete(drop = FALSE, name = NULL, guide = 'none') +
    ggplot2::scale_shape_manual(drop = FALSE, name = NULL, values = c(21:24)) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(
        angle = 65,
        hjust = 1,
        size = fntsize
      ),
      legend.position = 'bottom'
    ) +
    ggplot2::guides(fill = ggplot2::guide_legend(override.aes = list(size = 5)))

  gg2 %>%
    return()
}

test_that("plotLOAF errors", {
  empty_tibble <- tibble::tribble(
    ~name, ~value
  )
  expect_error(plotLOAF(empty_tibble))
  false_example1 <- relatedness_example %>% dplyr::rename(bob = pair)
  expect_error(plotLOAF(false_example1))
  expect_error(plotLOAF(kin_example, nsnps_cutoff = -1))
})

#' relatedness_example
#'
#' this is an example of the tibble made by callRelatedness()
#'
#' @format ## `relatedness_example`
#' A data frame with 15 rows and 13 columns:
#' \describe{
#' \item{row}{The row number}
#' \item{pair}{the pair of individuals that are compared.}
#' \item{relationship}{the highest posterior probability estimate of the degree of relatedness.}
#' \item{pmr}{the pairwise mismatch rate (mismatch/nsnps).}
#' \item{sd}{the estimated standard deviation of the pmr.}
#' \item{mismatch}{the number of sites which did not match for each pair.}
#' \item{nsnps}{the number of overlapping snps that were compared for each pair.}
#' \item{ave_re}{ the value for the background relatedness used for normalisation.}
#' \item{Same_Twins}{the posterior probability associated with a same individual/twins classification.}
#' \item{First_Degree}{the posterior probability associated with a first-degree classification.}
#' \item{Second_Degree}{the posterior probability associated with a second-degree classification.}
#' \item{Unrelated}{the posterior probability associated with an unrelated classification.}
#' \item{BF}{A strength of confidence in the Bayes Factor associated with the highest posterior probability classification compared to the 2nd highest.}
#' }
"relatedness_example"

utils::globalVariables(
  c("sex", "pop1", "pop2", "group")
)
#' plotDOUGH
#'
#' @param in_tibble A tibble containing the output from processEigenstrat().
#' @param indfile Path to an ind file with columns for the individuals IDs, genetic sex (not used) and population.
#' @param nsnps_cutoff Minimum number of overlapping sites for a pair to be included.
#' @param fntsize Font size for x axis tick labels.
#' @param verbose Controls the printing of progress to console.
#' @param removeBetween TRUE/FALSE for whether to include the “between population” PMR values as their own group (FALSE), or to exclude them (TRUE).
#'
#' @returns plots
#'
#' @export
plotDOUGH <- function(
  in_tibble,
  indfile,
  nsnps_cutoff = NULL,
  fntsize = 7,
  verbose = TRUE,
  removeBetween = FALSE
) {
  # Check in_tibble is real
  # Test that the in_tibble is of the correct form
  if (nrow(in_tibble) == 0) {
    stop('The input tibble is empty')
  }
  col_names_check <- c('pair', 'nsnps', 'mismatch', 'pmr')
  if (!all(col_names_check %in% names(in_tibble))) {
    stop(paste0(
      'Input tibble/data.frame must have 4 columns named: ',
      paste0(col_names_check, collapse = ', '),
      '.'
    ))
  }

  # Test that the overlapping snp cut off is sensible.
  if (is.null(nsnps_cutoff)) {
    nsnps_cutoff <- 5e2
    if (verbose) {
      cat(sprintf(
        'No minimum number of overlapping SNPs given.\nUsing default minimum of 500.\n'
      ))
    }
  }
  if ((nsnps_cutoff <= 0)) {
    stop(
      'The cut off for the number of overlapping snps must be greater than 0.'
    )
  } else if (nsnps_cutoff >= max(in_tibble$nsnps, na.rm = T)) {
    stop(
      'The cut off for the number of overlapping snps cannot be greater than the maximum number of overlapping SNPs that were observed!'
    )
  }
  # Check indfile exists
  if (!file.exists(indfile)) {
    stop(paste0('indfile ', indfile, ' does not exist!'))
  }

  IND <- readr::read_tsv(
    indfile,
    show_col_types = FALSE,
    col_names = c('ind', 'sex', 'pop')
  ) %>%
    dplyr::select(-sex)

  plot_data <- in_tibble %>%
    dplyr::filter(nsnps > nsnps_cutoff) %>%
    tidyr::separate(
      col = pair,
      into = c('ind1', 'ind2'),
      sep = ' - ',
      remove = FALSE
    ) %>%
    dplyr::left_join(IND, by = c('ind1' = 'ind')) %>%
    dplyr::left_join(IND, by = c('ind2' = 'ind'), suffix = c('1', '2')) %>%
    dplyr::mutate(group = ifelse(pop1 == pop2, pop1, 'Between Groups'))

  # Check that all individuals are accounted for
  fileInds <- c(plot_data$ind1, plot_data$ind2) %>%
    base::unique() %>%
    base::sort()

  missingInds <- base::setdiff(x = fileInds, y = IND$ind)

  if (length(missingInds) > 0) {
    missing_str <- missingInds %>%
      paste0(collapse = ', ') %>%
      sprintf(
        'The following individuals are in the BREADR analysis, but not the ind file: %s',
        .
      )
    stop(missing_str)
  }

  LEVELS <- plot_data$group %>%
    base::unique() %>%
    base::sort() %>%
    base::setdiff('Between Groups') %>%
    c(., 'Between Groups')

  outplot <- ggstatsplot::ggbetweenstats(
    data = plot_data %>%
      dplyr::mutate(group = factor(group, levels = LEVELS)) %>%
      dplyr::filter(
        group != ifelse(removeBetween, 'Between Groups', 'Can be Ignored Here')
      ),
    x = group,
    y = pmr,
    point.args = list(
      position = ggplot2::position_jitterdodge(dodge.width = 0.6),
      alpha = 0.5,
      size = 2,
      shape = 16
    ),
    type = 'nonparametric',
    plot.type = "box",
    drop = FALSE,
    pairwise.comparisons = TRUE,
    pairwise.display = "significant",
    centrality.plotting = TRUE,
    bf.message = FALSE,
    ggtheme = ggplot2::theme_bw(),
    xlab = 'Population',
    ylab = 'PMR'
  )
  return(outplot)
}

#' counts_example
#'
#' this is an example of the tibble made by processEigenstrat().
#'
#' @format ## `counts_example`
#' A data frame with 15 rows and 4 columns:
#' \describe{
#' \item{pair}{the pair of individuals that are compared}
#' \item{nsnps}{the number of overlapping snps that were compared for each pair.}
#' \item{mismatch}{the number of sites which did not match for each pair.}
#' \item{pmr}{the pairwise mismatch rate (mismatch/nsnps).}
#' }
"counts_example"

utils::globalVariables(c("deam", "pop", "chr",
                         "keep", "site", "nsnps",
                         "pmr", "mismatch", "ave_rel",
                         "Same_Twins", "First_Degree", "Second_Degree",
                         "Unrelated", "normConst", "relationship", "pair",
                         "n", "sd", "everything", "col_names_tibble",
                         "meanPMR", "ymin", "ymax", "model", "p.sim", "y",
                         "posterior", "label"))
getfilter <- function(x,gap){
  i <- 1
  res <- c(1)
  while(T){
    newMin <- which((x-x[i])>=gap)
    if(length(newMin)==0){
      break
    }else{
      res <- c(res,newMin%>%min())
      i <- newMin%>%min()
    }
  }
  return(1:length(x)%in%res)
}
weightedBinom <- function(x,N,p,lambda=10,uppern=1e3,log=F){
  pVec <- (1-0.5^(1:(uppern+1)))*p
  logP <- dtruncatedPoisson(0:uppern,d=2,lambda=lambda,log=T)+stats::dbinom(x,N,pVec,log=T)
  if(log){
    return(matrixStats::logSumExp(logP))
  }else{
    return(sum(exp(logP)))
  }
}
dtruncatedPoisson <- function(k,d,lambda,log=F){
  numerator <- k*log(lambda)
  denominator <- lfactorial(k)+log(exp(lambda)-sum(lambda^(0:d)/(factorial(0:d))))
  if(log){
    return(numerator-denominator+log(k%%1==0)+log(k>d))
  }else{
    return(exp(numerator-denominator)*(k%%1==0)*(k>d))
  }
}
ggcolorhue <- function(n) {
  hues = seq(15, 375, length = n + 1)
  grDevices::hcl(h = hues, l = 65, c = 100)[1:n]
}
convertP <- function(p,pbar){
  suppressWarnings(ifelse(p<pbar,log(1-p/pbar)/log(0.5)-1,Inf)) %>% return()
}
label_formatter <- function(x){
  label <- dplyr::case_when(
    x >= 0.01 ~ as.character(round(x, 4)),
    x < 1e-6 ~ "{}<1 %*% 10^-6",
    TRUE ~ stringr::str_replace(
      formatC(x, digits = 2, format = "e"),
      "e", " %*% 10^"
    )
  )
  return(label)
}

#' saveSLICES
#'
#' Plots all pairwise diagnostic plots (in a tibble as output by callRelatedness),
#' as produced by plotSLICE, to a folder.
#' Options include the width and height of the output files, and the units in
#' which these dimensions are measured.
#'
#' @param in_tibble a tibble that is the output of the callRelatedness() function.
#' @param outFolder the folder into which all diagnostic plots will be saved
#' @param width the width of the output PDFs.
#' @param height the height of the output PDFs.
#' @param units the units for the height and width of the output PDFs.
#' @param verbose Controls the printing of progress to console.
#'
#' @return nothing
#' @export
#'
#' @examples
#' \donttest{
#' saveSLICES(relatedness_example[1:3, ], outFolder=tempdir())
#' }
saveSLICES <- function(
  in_tibble,
  outFolder = NULL,
  width = 297,
  height = 210,
  units = 'mm',
  verbose = TRUE
) {
  # Test that the in_tibble is of the correct form
  if (nrow(in_tibble) == 0) {
    stop('The input tibble is empty')
  }
  col_names_check <- c(
    'row',
    'pair',
    'relationship',
    'pmr',
    'sd',
    'mismatch',
    'nsnps',
    'ave_rel',
    'Same_Twins',
    'First_Degree',
    'Second_Degree',
    'Unrelated'
  )
  if ((!all(col_names_check %in% names(in_tibble)))) {
    stop(paste0(
      'Input tibble/data.frame must have 4 columns named: ',
      paste0(col_names_check, collapse = ', '),
      '.'
    ))
  }

  # Test to see if outfolder is set and exists
  if (is.null(outFolder)) {
    stop('No output folder given.')
  } else {
    if (!dir.exists(outFolder)) {
      stop('Output folder does not exist.')
    }
  }

  # Make sure output folder string ends in forward slash
  if (stringr::str_sub(outFolder, -1) != '/') {
    outFolder <- paste0(outFolder, '/')
  }

  # Test if plotting dimensions are numeric, and if the units are valid.
  if (any(!sapply(c(width, height), is.numeric))) {
    stop('Height and width must be numeric values')
  }
  if (!units %in% c(c("in", "cm", "mm", "px"))) {
    stop("Plot dimension units must be 'in', 'cm', 'mm' or 'px'")
  }

  # Start plotting!
  if (verbose) {
    cat('Starting pairwise plot creation.\n')
  }

  pb = utils::txtProgressBar(
    min = 1,
    max = nrow(in_tibble),
    initial = 1,
    style = 3
  )
  for (i in 1:nrow(in_tibble)) {
    v.sm <- plotSLICE(in_tibble, i, showPlot = F)
    ggplot2::ggsave(
      v.sm,
      filename = paste0(outFolder, gsub(' - ', '_', in_tibble$pair[i]), '.pdf'),
      width = width,
      height = height,
      units = units
    )
    utils::setTxtProgressBar(pb, i)
  }
  if (verbose) {
    cat(paste0('\nCompleted\nAll plots in: ', outFolder, '\n'))
  }
  return(TRUE)
}

get_column_old <- function(genofile, col = 1){
  system(paste0('cut -c ',col,' ',genofile),intern=T)
}
#' get column
#'
#' @param genofile genofile
#' @param col column to return
#'
#' @return column of numbers
#' @export
get_column_new <- function(genofile, col = 1){
  readr::read_lines(genofile) |>
    purrr::map(\(x) stringr::str_sub(x, col, col)) |>
    unlist()
}
# pacman::p_load(conflicted, tidyverse, targets, data.table)
# genofile <- system.file("extdata", "example.geno.txt", package = "BREADR")
# identical(get_column_old(genofile, 1), get_column_new(genofile, 1))
#
# microbenchmark::microbenchmark(
#   get_column_old(genofile, 1),
#   get_column_new(genofile, 1)
# )

#' process Eigenstrat data
#'
#'A function that takes paths to an eigenstrat trio (ind, snp and geno file) and
#'returns the pairwise mismatch rate for all pairs on a thinned set of SNPs.
#'Options include choosing thinning parameter, subsetting by population names,
#'and filtering out SNPs for which deamination is possible.
#'
#' @param indfile path to eigenstrat ind file
#' @param genofile path to eigenstrat geno file.
#' @param snpfile path to eigenstrat snp file.
#' @param filter_length the minimum distance between sites to be compared (to reduce the effect of LD).
#' @param pop_pattern a character vector of population names to filter the ind file if only some populations are to compared.
#' @param filter_deam a TRUE/FALSE for if C->T and G->A sites should be ignored.
#' @param outfile (OPTIONAL) a path and filename to which we can save the output of the function as a TSV,
#' if NULL, no back up saved. If no outfile, then a tibble is returned.
#' @param chromosomes the chromosome to filter the data on.
#' @param verbose controls printing of messages to console
#'
#' @return out_tibble: A tibble containing four columns:
# - pair [char]: the pair of individuals that are compared.
# - nsnps [numeric]: the number of overlapping snps that were compared for each pair.
# - mismatch [numeric]: the number of sites which did not match for each pair.
# - pmr [numeric]: the pairwise mismatch rate (mismatch/nsnps).
#' @export
#'
#' @examples
#' # Use internal files to the package as an example
#' indfile <- system.file("extdata", "example.ind.txt", package = "BREADR")
#' genofile <- system.file("extdata", "example.geno.txt", package = "BREADR")
#' snpfile <- system.file("extdata", "example.snp.txt", package = "BREADR")
#' processEigenstrat_old(
#' indfile, genofile, snpfile,
#' filter_length=1e5,
#' pop_pattern=NULL,
#' filter_deam=FALSE
#' )
processEigenstrat_old <- function(indfile, genofile, snpfile,
                              filter_length=NULL, pop_pattern=NULL, filter_deam=FALSE,
                              outfile=NULL, chromosomes=NULL, verbose = TRUE){
  # Check to see if eigenstrat files exist
  if(!file.exists(indfile)){
    stop(paste0('indfile ',indfile,' does not exist!'))
  }
  if(!file.exists(genofile)){
    stop(paste0('genofile ',genofile,' does not exist!'))
  }
  if(!file.exists(snpfile)){
    stop(paste0('snpfile ',snpfile,' does not exist!'))
  }

  # Just check to see if a length filter was included and is reasonable, and if not,
  # let the user know of the default and/or problem.
  if(is.null(filter_length)){
    filter_length <- 1e5
    if(verbose){
      cat(sprintf('No site distance filter.\nUsing default minimum of 1e5.\n'))
    }
  }
  if(!is.numeric(filter_length)|(filter_length<=0)){
    stop('filter_length must be a positive number.')
  }

  # Check if all pop_names are in the possible populations.
  # Change to delim so works in different countries.
  # ind_raw <- readr::read_delim(
  #   indfile,
  #   col_names=c('ind','sex','pop'),
  #   col_types=c('ccc'),
  #   delim = "\t"
  # )
  ind_raw <- read_ind(indfile)
  if(!all(pop_pattern%in%ind_raw$pop)){
    paste0('Following populations in pop_pattern do not exist in indfile: ',
           paste0(setdiff(pop_pattern,ind_raw$pop),collapse=', ')) %>%
      stop()
  }

  # Check that filter deam is a logical variable.
  if(!is.logical(filter_deam)){
    stop('filter_deam must be a logical (boolean) variable, i.e., TRUE or FALSE.')
  }

  # Check that the folder in which we plan to save the outfile actually exists.
  if(!is.null(outfile)){
    if(!file.exists(dirname(outfile))){
      stop(paste0('Output folder ',dirname(outfile),' does not exist!'))
    }
  }

  # Collect SNP details
  if(verbose){
    cat('Reading in SNP data.\n')
  }
  ## Read in SNP file
  snps <- read_snp(snpfile)
  ## Add relative position
  snps <-
    snps %>%
    dplyr::mutate(
      relative_pos=1:dplyr::n()
    )
  ## Mutate and filter for DEAM
  if(filter_deam){
    snps <-
      snps %>%
      dplyr::mutate(
        deam=dplyr::case_when(
          anc=='C'&der=='T' ~ T,
          anc=='G'&der=='A' ~ T,
          T ~ F)
      ) %>%
      dplyr::filter(!deam) %>%
      dplyr::select(-deam)
  }
  # if(filter_deam){
  #   snps <- readr::read_delim(
  #     snpfile,
  #     col_names=c('snp','chr','pos','site','anc','der'),
  #     col_types=c('cdddcc'),
  #     delim = "\t"
  #   ) %>%
  #     dplyr::mutate(relative_pos=1:dplyr::n()) %>%
  #     dplyr::mutate(deam=dplyr::case_when(anc=='C'&der=='T' ~ T,
  #                                  anc=='G'&der=='A' ~ T,
  #                                  T ~ F)) %>%
  #     dplyr::filter(!deam) %>%
  #     dplyr::select(-deam)
  # }else{
  #   snps <- readr::read_delim(
  #     snpfile,
  #     col_names=c('snp','chr','pos','site','anc','der'),
  #     col_types=c('cdddcc'),
  #     delim = "\t"
  #   ) %>%
  #     dplyr::mutate(relative_pos=1:dplyr::n())
  # }
  # Filter for requested chromosomes
  available_chr <- snps$chr %>%
    unique()
  if(!is.null(chromosomes)){
    if(!any(chromosomes%in%available_chr)){
      stop(paste0('Chromosome names do not match SNP file:\n',
                  'Requested: ',paste0(chromosomes,collapse=', '),'\n',
                  'Available: ',paste0(available_chr,collapse=', '),'\n'))
    }else{
      snps <- snps %>%
        dplyr::filter(chr%in%chromosomes)
    }
  }
  # Print information about chromosomes
  if(verbose){
    cat(paste0('Analysing chromosomes:\n',paste0(unique(snps$chr),collapse=', '),'\n'))
  }
  # Collect (potentially filtered) ind details
  if(is.null(pop_pattern)){
    ind <-  ind_raw %>%
      dplyr::mutate(row_number=1:dplyr::n())
  }else{
    ind <- ind_raw %>%
      dplyr::mutate(row_number=1:dplyr::n()) %>%
      dplyr::filter(pop%in%pop_pattern)
  }

  # Throw error if ind has no rows.
  n_ind <- nrow(ind)
  if(n_ind<=1){
    if(is.null(pop_pattern)){
      stop(paste0(n_ind,' individuals in final, filtered ind file. Check ind file.'))
    }else{
      stop(paste0(n_ind,' individuals in final, filtered ind file. Check ind file, and check pop_pattern.'))
    }
  }
  geno_list <- vector(mode='list',length=n_ind)

  if(verbose){
    cat('Starting to read in genotype data.\n')
  }
  pb1 = utils::txtProgressBar(min=1,max=length(geno_list),initial=1,style=3)
  for(i in 1:n_ind){
    geno_list[[i]] <- (system(paste0('cut -c ',ind$row_number[i],' ',genofile),intern=T) %>% dplyr::na_if("9"))[snps$relative_pos]
    utils::setTxtProgressBar(pb1,i)

  }
  if(verbose){
    cat('  Complete.\n\n')

    cat('Starting to compare genotypes and calculate PMR.\n')
  }
  counter <- 1
  out_tibble <- tibble::tibble(pair=rep('x',choose(n_ind,2)),nsnps=0,mismatch=0,pmr=0)
  pb2 = utils::txtProgressBar(min=1,max=choose(n_ind,2),initial=1,style=3)
  for(i in 1:(n_ind-1)){
    ind_i <- ind$ind[i]
    vi <- geno_list[[i]]
    for(j in (i+1):n_ind){
      ind_j <- ind$ind[j]
      vj <- geno_list[[j]]

      snps_filtered_prefiltered <- snps  %>%
        dplyr::mutate(row_num=1:dplyr::n()) %>%
        dplyr::filter(!is.na(vi),!is.na(vj))

      if(nrow(snps_filtered_prefiltered)>0){
        snps_filtered <- snps_filtered_prefiltered %>%
          dplyr::group_by(chr) %>%
          dplyr::mutate(keep=getfilter(site,filter_length)) %>%
          dplyr::ungroup() %>%
          dplyr::filter(keep) %>%
          dplyr::select(-keep)

        out_tibble$pair[counter] <- paste0(ind_i,' - ',ind_j)
        out_tibble$nsnps[counter] <- (!is.na(vi[snps_filtered$row_num])&!is.na(vj[snps_filtered$row_num])) %>% sum()
        out_tibble$mismatch[counter] <- (vi[snps_filtered$row_num]!=vj[snps_filtered$row_num]) %>% sum(na.rm=T)
        out_tibble$pmr[counter] <-  out_tibble$mismatch[counter]/out_tibble$nsnps[counter]
      }else{
        out_tibble$pair[counter] <- paste0(ind_i,' - ',ind_j)
        out_tibble$nsnps[counter] <- 0
        out_tibble$mismatch[counter] <- 0
        out_tibble$pmr[counter] <-  NA
      }
      counter <- counter+1
      utils::setTxtProgressBar(pb2,counter)
    }
  }

  if(!is.null(outfile)){
    if(verbose){
      cat(sprintf('\n\nSaving processed data to %s.\n',outfile))
    }
    readr::write_delim(out_tibble,file=outfile,delim='\t')
  }
  if(verbose){
    cat('\nComplete.\n\n')
  }

  out_tibble %>% return()
}

#' plotSLICE
#'
#' A function for plotting the diagnostic information when classifying a
#' specific pair (defined by the row number or pair name) of individuals.
#' Output includes the PDFs for each degree of relatedness (given the number of
#' overlapping SNPs) in panel A, and the normalised posterior probabilities
#' for each possible degree of relatedness.
#'
#' @param in_tibble a tibble that is the output of the callRelatedness() function.
#' @param row either the row number or pair name for which the
#' posterior distribution is to be plotted.
#' @param title an optional title for the plot. If NULL, the pair
#' from the user-defined row is used.
#' @param class_prior the prior probabilities for same/twin, 1st-degree,
#' 2nd-degree, unrelated, respectively.
#' @param showPlot If TRUE, display plot. If FALSE, just pass plot as a variable.
#' @param which_plot if 1, returns just the plot of the posterior distributions,
#' if 2 returns just the normalised posterior values. Anything else returns both
#' plots.
#' @param labels a length two character vector of labels for plots. Default is
#' no labels.
#'
#' @return a two-panel diagnostic ggplot object
#' @export
#'
#' @examples
#' plotSLICE(relatedness_example, row=1)
plotSLICE <- function(
  in_tibble,
  row,
  title = NULL,
  class_prior = rep(1 / 4, 4),
  showPlot = TRUE,
  which_plot = 0,
  labels = NULL
) {
  # Test that the in_tibble is of the correct form
  if (nrow(in_tibble) == 0) {
    stop('The input tibble is empty')
  }
  col_names_check <- c(
    'row',
    'pair',
    'relationship',
    'pmr',
    'sd',
    'mismatch',
    'nsnps',
    'ave_rel',
    'Same_Twins',
    'First_Degree',
    'Second_Degree',
    'Unrelated'
  )
  if (!all(col_names_check %in% names(in_tibble))) {
    stop(paste0(
      'Input tibble/data.frame must have 12 columns named: ',
      paste0(col_names_check, collapse = ', '),
      '.'
    ))
  }

  # check row is a numeric or char variable, and if char, find the row.
  if (is.character(row)) {
    rowtmp <- row
    row <- which(row == in_tibble$pair)
    if (length(row) == 0) {
      stop(paste0('There is no pair called: ', rowtmp))
    }
  } else if (is.numeric(row) & (row > nrow(in_tibble))) {
    stop(
      'row number requested was higher than the number of rows in input tibble!'
    )
  } else if (!is.numeric(row)) {
    stop('row must be a number or a pair name.')
  }

  # Check that the prior distribution makes sense:
  if (any(class_prior < 0) | any(class_prior > 1)) {
    stop(
      'Posterior probabilities for degrees of relatedness must be values between 0 and 1.'
    )
  } else if (sum(class_prior) != 1) {
    stop('Posterior probabilities for degrees of relatedness must sum to 1.')
  }

  # Check that showPlot is a logical variable
  if (!is.logical(showPlot)) {
    stop('showPlot must be a logical (TRUE/FALSE) variable.')
  }

  # Check that labels is length two is present
  if (!is.null(labels)) {
    stopifnot(length(labels) == 2)
  }

  # grab PMR info
  x <- in_tibble$mismatch[row]
  N <- in_tibble$nsnps[row]
  M <- in_tibble$ave_rel[row]

  # Set title if NULL
  if (is.null(title)) {
    title <- in_tibble$pair[row]
  }

  # Set max number of points which are plotted at 5000 (uniformly). Set for efficiency.
  if (N > 5e3) {
    support <- seq(0, N, length.out = 5e3) %>% round()
  } else {
    support <- 0:N
  }

  # Find limit of x-axis.
  XLIM <- (M) + 5 * sqrt((M) * (1 - M) / N)

  # Define possible classes.
  kclasses <- c('Same_Twins', 'First_Degree', 'Second_Degree', 'Unrelated')

  # find pdf values across "support" for each relatedness category
  distribution.tib <- tibble::tibble(
    x = rep(support, 4),
    p.sim = rep((1 - 0.5^c(1:3, Inf)) * M, each = length(support)),
    model = rep(kclasses, each = length(support))
  ) %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      y = (model == 'Same_Twins' | model == 'Second_Degree') *
        stats::dbinom(x, N, p.sim) +
        (model == 'First_Degree') * stats::dbinom(x, N, p.sim) + #dbbinom(x,N,alpha,beta)+
        (model == 'Unrelated') * weightedBinom(x, N, p.sim),
      model = factor(model, levels = kclasses)
    )

  # Define plot height for observed PMR below plot
  h <- max(distribution.tib$y) * 0.05

  # Find highest probability points for dashed lines
  midpoint.tib <- distribution.tib %>%
    dplyr::group_by(model) %>%
    dplyr::filter(y == max(y))

  # find critical value
  zstar <- stats::qnorm(0.975)

  # make plot of probability curves
  distribution.plot <- distribution.tib %>%
    ggplot2::ggplot(ggplot2::aes(x = x / N, y = y)) +
    ggplot2::scale_x_continuous(limits = c(0, XLIM), expand = c(0, 0)) +
    ggplot2::scale_y_continuous(
      limits = c(-h, max(distribution.tib$y)),
      expand = c(0, 0)
    ) +
    ggplot2::annotate(
      geom = 'point',
      x = x / N,
      y = -h / 2,
      pch = 19,
      size = 2
    ) +
    ggplot2::geom_segment(
      x = x / N - zstar * sqrt((x / N) * (1 - x / N) / N),
      xend = x / N + zstar * sqrt((x / N) * (1 - x / N) / N),
      y = -h / 2,
      yend = -h / 2
    ) +
    ggplot2::geom_segment(
      x = x / N - zstar * sqrt((x / N) * (1 - x / N) / N),
      xend = x / N - zstar * sqrt((x / N) * (1 - x / N) / N),
      y = -h / 4,
      yend = -3 * h / 4
    ) +
    ggplot2::geom_segment(
      x = x / N + zstar * sqrt((x / N) * (1 - x / N) / N),
      xend = x / N + zstar * sqrt((x / N) * (1 - x / N) / N),
      y = -h / 4,
      yend = -3 * h / 4
    ) +
    ggplot2::theme_bw() +
    ggplot2::xlab(paste0('Pairwise Mismatch Rate (n=', N, ')')) +
    ggplot2::ylab(NULL) +
    ggplot2::theme(legend.position = 'bottom') +
    ggplot2::geom_ribbon(
      ggplot2::aes(ymin = 0, ymax = y, fill = model),
      col = 'black',
      alpha = 0.5
    ) +
    ggplot2::geom_segment(
      data = midpoint.tib,
      ggplot2::aes(x = p.sim, xend = p.sim, y = 0, yend = y * 0.99),
      linetype = 'dashed',
      col = 'black'
    )

  # return just distribution plot is asked for
  if (which_plot == 1) {
    return(distribution.plot)
  }
  # Calculate and plot posterior probability values
  posterior.tibble <- tibble::tibble(
    posterior = c(
      stats::dbinom(x, N, (1 - 0.5^c(1:3)) * M),
      weightedBinom(x, N, M)
    ) *
      class_prior /
      sum(
        c(stats::dbinom(x, N, (1 - 0.5^c(1:3)) * M), weightedBinom(x, N, M)) *
          class_prior
      ),
    model = factor(kclasses, levels = kclasses)
  )
  posterior.tibble <- posterior.tibble %>%
    dplyr::mutate(
      label = label_formatter(posterior)
    )
  # return(posterior.tibble)
  posterior.plot <- posterior.tibble %>%
    ggplot2::ggplot(ggplot2::aes(x = model, y = posterior)) +
    ggplot2::theme_bw() +
    ggplot2::geom_bar(
      stat = 'identity',
      ggplot2::aes(fill = model),
      col = 'black',
      alpha = 0.8
    ) +
    ggplot2::geom_label(
      ggplot2::aes(
        label = label,
        colour = model
      ),
      parse = TRUE
    ) +
    ggplot2::ylab("Posterior Probability") +
    ggplot2::xlab('Model') +
    ggplot2::scale_x_discrete(guide = ggplot2::guide_axis(n.dodge = 2)) +
    ggplot2::theme(
      panel.grid.major = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      panel.background = ggplot2::element_blank(),
      axis.line = ggplot2::element_line(colour = "black")
    )
  # Return just posterior plot is asked for
  if (which_plot == 2) {
    return(posterior.plot)
  }
  # Combine plots
  # Old code left in as archaeology. Why would you not in a paper for relationship between samples.
  # gg1 <- gridExtra::arrangeGrob(distribution.plot+ggplot2::theme(legend.position='none'),
  #                               posterior.plot+ggplot2::theme(legend.position='none'),
  #                               nrow=1,
  #                               top=grid::textGrob(title,gp=grid::gpar(fontsize=20,font=3)))
  plot_grid <- ggpubr::ggarrange(
    distribution.plot,
    posterior.plot,
    nrow = 1,
    legend = "none",
    labels = labels
  )
  plot_grid <- ggpubr::annotate_figure(
    plot_grid,
    top = ggpubr::text_grob(title, size = 20)
  )
  # Plot if required
  if (showPlot) {
    return(plot_grid)
  }
  invisible(plot_grid)
}

test_that("plotSLICE errors", {
  empty_tibble <- tibble::tribble(
    ~name, ~value
  )
  expect_error(plotSLICE(empty_tibble))
  false_example1 <- relatedness_example %>% dplyr::rename(bob = pair)
  expect_error(plotSLICE(false_example1))
  expect_error(plotSLICE(relatedness_example))
  expect_error(plotSLICE(relatedness_example, row = 100))
  expect_error(plotSLICE(relatedness_example, row = 1, labels = "a"))
})

#' read_ind
#'
#' @param filename a IND text file.
#'
#' @return tibble with column headings: ind (CHR), sex (CHR), pop (CHR)
#' @export
#'
#' @examples
#' ind_snpfile <- system.file("extdata", "example.ind.txt", package = "BREADR")
#' read_ind(ind_snpfile)
read_ind <- function(filename){
    df <- data.table::fread(
      filename, header = FALSE,
      col.names=c("ind", "sex", "pop"),
      colClasses=list(character=1:3)
    )
    df <- tibble::as_tibble(df)
    return(df)
}

utils::globalVariables(
  c(
    "prior_Same_Twins",
    "prior",
    "degree",
    "p",
    "prior_First_Degree",
    "prior_Second_Degree",
    "case",
    "prior_Unrelated"
  )
)
#' priorSensitivity
#'
#' @param in_BREADR A tibble containing the output of callRelatedness().
#' @param row Either the row number or pair name for which the sensitivity analysis should be run.
#' @param degrees A vector of integers identifying which degrees of relatedness degrees to plot. Same/Twins (1), First-Degree (2), Second-Degree (3) and Unrelated (4).
#' @param grid_space The space between prior probability values that are tested (smaller values mean a finer grid size).
#' @param maxPrior Maximum value a prior probability can take (the right-hand limit of the x-axis).
#'
#' @returns plot
#'
#' @export
priorSensitivity <- function(
  in_BREADR,
  row,
  degrees = NULL,
  grid_space = 0.05,
  maxPrior = 0.5
) {
  # Test that the in_tibble is of the correct form
  if (base::nrow(in_BREADR) == 0) {
    stop('The input tibble is empty')
  }
  col_names_check <- c(
    'row',
    'pair',
    'relationship',
    'pmr',
    'sd',
    'mismatch',
    'nsnps',
    'ave_rel',
    'Same_Twins',
    'First_Degree',
    'Second_Degree',
    'Unrelated'
  )
  if ((!base::all(col_names_check %in% names(in_BREADR)))) {
    stop(base::paste0(
      'Input tibble/data.frame must have columns named: ',
      paste0(col_names_check, collapse = ', '),
      '.'
    ))
  }
  if (
    !(base::all(
      degrees %in% c('Same_Twins', 'First_Degree', 'Second_Degree', 'Unrelated')
    ) |
      all(degrees %in% (1:4)))
  ) {
    stop(
      'degrees must be a vector of either:\n1. Numbers from c(1,2,3,4)\n2.Strings from c(Same_Twins,First_Degree,Second_Degree,Unrelated)'
    )
  }
  if (!base::is.numeric(grid_space) | (grid_space <= 0) | (grid_space > 0.5)) {
    stop('grid_space must be a number between 0 and 0.5')
  }
  if (
    !base::is.numeric(maxPrior) |
      (maxPrior <= 0) |
      (maxPrior < grid_space) |
      maxPrior <= 0.25
  ) {
    stop('maxPrior must be a number between 0.25 and grid_space')
  }
  # check row is a numeric or char variable, and if char, find the row.
  if (base::is.character(row)) {
    rowtmp <- row
    row <- base::which(row == in_BREADR$pair)
    if (base::length(row) == 0) {
      stop(base::paste0('There is no pair called: ', rowtmp))
    }
  } else if (base::is.numeric(row) & (row > base::nrow(in_BREADR))) {
    stop(
      'row number requested was higher than the number of rows in input tibble!'
    )
  } else if (!base::is.numeric(row)) {
    stop('row must be a number or a pair name.')
  }

  hues <- base::seq(15, 375, length = 5)
  PAL <- grDevices::hcl(h = hues, l = 65, c = 100)[1:4]
  CALLS <- c('Same_Twins', 'First_Degree', 'Second_Degree', 'Unrelated')
  if (base::is.null(degrees)) {
    degrees <- CALLS
  }
  if (base::is.numeric(degrees)) {
    degrees <- CALLS[degrees]
  }

  grid_in <- base::seq(
    from = grid_space,
    to = min(1 - 3 * grid_space, maxPrior),
    by = grid_space
  )
  grid_check <- base::list(
    Same_Twins = grid_in,
    First_Degree = grid_in,
    Second_Degree = grid_in,
    Unrelated = grid_in
  ) %>%
    base::expand.grid() %>%
    tidyr::as_tibble() %>%
    dplyr::filter(
      abs(Same_Twins + First_Degree + Second_Degree + Unrelated - 1) < 1e-6
    ) %>%
    dplyr::arrange(Same_Twins, First_Degree, Second_Degree, Unrelated) %>%
    dplyr::rename_with(function(x) {
      return(paste0('prior_', x))
    })

  PAIR <- in_BREADR$pair[row]

  base::sprintf(
    'Starting sensitivity analysis (%i separate analyses)....\n"',
    nrow(grid_check)
  ) %>%
    cat()
  pb <- utils::txtProgressBar(
    min = 1,
    max = nrow(grid_check),
    initial = 1,
    style = 3
  )
  for (i in 1:base::nrow(grid_check)) {
    if (i == 1) {
      res_list <- base::vector(mode = 'list', length = base::nrow(grid_check))
    }
    cur_prior <- grid_check %>%
      dplyr::slice(i) %>%
      base::unlist() %>%
      base::as.vector()
    res_list[[i]] <- callRelatedness(
      in_BREADR %>%
        dplyr::select(pair, nsnps, mismatch, pmr),
      average_relatedness = in_BREADR$ave_rel,
      class_prior = cur_prior
    ) %>%
      dplyr::select(Same_Twins, First_Degree, Second_Degree, Unrelated) %>%
      dplyr::slice(row)
    utils::setTxtProgressBar(pb, i)
  }
  cat("\nCollating results....\n")
  ST <- res_list %>%
    dplyr::bind_rows() %>%
    dplyr::bind_cols(grid_check, .) %>%
    dplyr::select(
      prior_Same_Twins,
      Same_Twins,
      First_Degree,
      Second_Degree,
      Unrelated
    ) %>%
    dplyr::rename(prior = prior_Same_Twins) %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      call = CALLS[which.max(c(
        Same_Twins,
        First_Degree,
        Second_Degree,
        Unrelated
      ))]
    ) %>%
    dplyr::group_by(prior) %>%
    dplyr::summarise(
      Same_Twins = sum(call == 'Same_Twins') / dplyr::n(),
      First_Degree = sum(call == 'First_Degree') / dplyr::n(),
      Second_Degree = sum(call == 'Second_Degree') / dplyr::n(),
      Unrelated = sum(call == 'Unrelated') / dplyr::n()
    ) %>%
    dplyr::ungroup() %>%
    tidyr::gather(degree, p, 2:5) %>%
    dplyr::mutate(case = 'Same_Twins')
  FD <- res_list %>%
    dplyr::bind_rows() %>%
    dplyr::bind_cols(grid_check, .) %>%
    dplyr::select(
      prior_First_Degree,
      Same_Twins,
      First_Degree,
      Second_Degree,
      Unrelated
    ) %>%
    dplyr::rename(prior = prior_First_Degree) %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      call = CALLS[which.max(c(
        Same_Twins,
        First_Degree,
        Second_Degree,
        Unrelated
      ))]
    ) %>%
    dplyr::group_by(prior) %>%
    dplyr::summarise(
      Same_Twins = sum(call == 'Same_Twins') / dplyr::n(),
      First_Degree = sum(call == 'First_Degree') / dplyr::n(),
      Second_Degree = sum(call == 'Second_Degree') / dplyr::n(),
      Unrelated = sum(call == 'Unrelated') / dplyr::n()
    ) %>%
    dplyr::ungroup() %>%
    tidyr::gather(degree, p, 2:5) %>%
    dplyr::mutate(case = 'First_Degree')
  SD <- res_list %>%
    dplyr::bind_rows() %>%
    dplyr::bind_cols(grid_check, .) %>%
    dplyr::select(
      prior_Second_Degree,
      Same_Twins,
      First_Degree,
      Second_Degree,
      Unrelated
    ) %>%
    dplyr::rename(prior = prior_Second_Degree) %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      call = CALLS[which.max(c(
        Same_Twins,
        First_Degree,
        Second_Degree,
        Unrelated
      ))]
    ) %>%
    dplyr::group_by(prior) %>%
    dplyr::summarise(
      Same_Twins = sum(call == 'Same_Twins') / dplyr::n(),
      First_Degree = sum(call == 'First_Degree') / dplyr::n(),
      Second_Degree = sum(call == 'Second_Degree') / dplyr::n(),
      Unrelated = sum(call == 'Unrelated') / dplyr::n()
    ) %>%
    dplyr::ungroup() %>%
    tidyr::gather(degree, p, 2:5) %>%
    dplyr::mutate(case = 'Second_Degree')
  UR <- res_list %>%
    dplyr::bind_rows() %>%
    dplyr::bind_cols(grid_check, .) %>%
    dplyr::select(
      prior_Unrelated,
      Same_Twins,
      First_Degree,
      Second_Degree,
      Unrelated
    ) %>%
    dplyr::rename(prior = prior_Unrelated) %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      call = CALLS[which.max(c(
        Same_Twins,
        First_Degree,
        Second_Degree,
        Unrelated
      ))]
    ) %>%
    dplyr::group_by(prior) %>%
    dplyr::summarise(
      Same_Twins = sum(call == 'Same_Twins') / dplyr::n(),
      First_Degree = sum(call == 'First_Degree') / dplyr::n(),
      Second_Degree = sum(call == 'Second_Degree') / dplyr::n(),
      Unrelated = sum(call == 'Unrelated') / dplyr::n()
    ) %>%
    dplyr::ungroup() %>%
    tidyr::gather(degree, p, 2:5) %>%
    dplyr::mutate(case = 'Unrelated')

  out.gg.dat <- list(ST, FD, SD, UR) %>%
    dplyr::bind_rows() %>%
    dplyr::filter(case %in% degrees) %>%
    dplyr::mutate(
      case = case %>%
        factor(
          levels = c(
            'Same_Twins',
            'First_Degree',
            'Second_Degree',
            'Unrelated'
          ),
          labels = c('Same/Twins', '1st-Degree', '2nd-Degree', 'Unrelated')
        ),
      degree = degree %>%
        factor(
          levels = c(
            'Same_Twins',
            'First_Degree',
            'Second_Degree',
            'Unrelated'
          ),
          labels = c('Same/Twins', '1st-Degree', '2nd-Degree', 'Unrelated')
        )
    )
  out.gg <- out.gg.dat %>%
    ggplot2::ggplot(ggplot2::aes(
      x = prior,
      y = p * 100,
      alpha = p,
      fill = degree
    )) +
    ggplot2::theme_bw() +
    ggplot2::geom_bar(stat = 'identity', col = 'black') +
    ggplot2::geom_hline(yintercept = 50, linetype = 'dashed') +
    ggplot2::facet_wrap(ggplot2::vars(case), nrow = base::length(degrees)) +
    ggplot2::xlab('Prior Probability') +
    ggplot2::ylab('Highest Posterior Calls (%)') +
    ggplot2::scale_y_continuous(breaks = base::seq(0, 100, by = 20)) +
    ggplot2::scale_fill_discrete(name = 'Degree') +
    ggplot2::scale_alpha_continuous(guide = 'none') +
    ggplot2::coord_cartesian(expand = FALSE) +
    ggplot2::ggtitle(PAIR) +
    ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0.5))

  base::cat('Done.\n')
  return(out.gg)
}

utils::globalVariables(
  c("digit", "snp", "N", ".")
)
#' process Eigenstrat data - alternative version
#'
#'A function that takes paths to an eigenstrat trio (ind, snp and geno file) and
#'returns the pairwise mismatch rate for all pairs on a thinned set of SNPs.
#'Options include choosing thinning parameter, subsetting by population names,
#'and filtering out SNPs for which deamination is possible.
#'
#' @param indfile path to eigenstrat ind file
#' @param genofile path to eigenstrat geno file.
#' @param snpfile path to eigenstrat snp file.
#' @param filter_length the minimum distance between sites to be compared (to reduce the effect of LD).
#' @param pop_pattern a character vector of population names to filter the ind file if only some populations are to compared.
#' @param filter_deam a TRUE/FALSE for if C->T and G->A sites should be ignored.
#' @param outfile (OPTIONAL) a path and filename to which we can save the output of the function as a TSV,
#' if NULL, no back up saved. If no outfile, then a tibble is returned.
#' @param chromosomes the chromosome to filter the data on.
#' @param verbose controls printing of messages to console
#' @param byPop Boolean variable identifying if all pairs of individuals should be compared (FALSE), or just pairs of individuals in the same populations (TRUE)
#' @param minMerge (integer) the minimum group size (smaller groups are merged).
#'
#' @return out_tibble: A tibble containing four columns:
# - pair [char]: the pair of individuals that are compared.
# - nsnps [numeric]: the number of overlapping snps that were compared for each pair.
# - mismatch [numeric]: the number of sites which did not match for each pair.
# - pmr [numeric]: the pairwise mismatch rate (mismatch/nsnps).
#' @import data.table
#' @export
#'
#' @examples
#' # Use internal files to the package as an example
#' indfile <- system.file("extdata", "example.ind.txt", package="BREADR")
#' genofile <- system.file("extdata", "example.geno.txt", package="BREADR")
#' snpfile <- system.file("extdata", "example.snp.txt", package="BREADR")
#' processEigenstrat(
#' indfile, genofile, snpfile,
#' filter_length=1e5,
#' pop_pattern=NULL,
#' filter_deam=FALSE
#' )
processEigenstrat <- function(
  indfile,
  genofile,
  snpfile,
  filter_length = NULL,
  pop_pattern = NULL,
  filter_deam = FALSE,
  outfile = NULL,
  chromosomes = NULL,
  verbose = TRUE,
  byPop = FALSE,
  minMerge = 2
) {
  # Check to see if eigenstrat files exist
  if (!file.exists(indfile)) {
    stop(paste0('indfile ', indfile, ' does not exist!'))
  }
  if (!file.exists(genofile)) {
    stop(paste0('genofile ', genofile, ' does not exist!'))
  }
  if (!file.exists(snpfile)) {
    stop(paste0('snpfile ', snpfile, ' does not exist!'))
  }

  # minMerge should be a number ≥2
  if ((!is.numeric(minMerge)) | (minMerge < 2)) {
    stop('minMerge must be a number >= 2.')
  }

  # Just check to see if a length filter was included and is reasonable, and if not,
  # let the user know of the default and/or problem.
  if (is.null(filter_length)) {
    filter_length <- 1e5
    if (verbose) {
      cat(sprintf('No site distance filter.\nUsing default minimum of 1e5.\n'))
    }
  }
  if (!is.numeric(filter_length) | (filter_length <= 0)) {
    stop('filter_length must be a positive number.')
  }

  # Check if all pop_names are in the possible populations.
  # Change to delim so works in different countries.
  ind_raw <- read_ind(indfile)
  if (!all(pop_pattern %in% ind_raw$pop)) {
    paste0(
      'Following populations in pop_pattern do not exist in indfile: ',
      paste0(setdiff(pop_pattern, ind_raw$pop), collapse = ', ')
    ) %>%
      stop()
  }

  # Check that filter deam is a logical variable.
  if (!is.logical(filter_deam)) {
    stop(
      'filter_deam must be a logical (boolean) variable, i.e., TRUE or FALSE.'
    )
  }

  # Check that the folder in which we plan to save the outfile actually exists.
  if (!is.null(outfile)) {
    if (!file.exists(dirname(outfile))) {
      stop(paste0('Output folder ', dirname(outfile), ' does not exist!'))
    }
  }

  # Collect SNP details
  if (verbose) {
    cat('Reading in SNP data.\n')
  }
  ## Read in SNP file
  snps <- read_snp(snpfile)
  ## Add relative position
  snps <-
    snps %>%
    dplyr::mutate(
      relative_pos = 1:dplyr::n()
    )
  ## Mutate and filter for DEAM
  if (filter_deam) {
    snps <-
      snps %>%
      dplyr::mutate(
        deam = dplyr::case_when(
          anc == 'C' & der == 'T' ~ T,
          anc == 'G' & der == 'A' ~ T,
          T ~ F
        )
      ) %>%
      dplyr::filter(!deam) %>%
      dplyr::select(-deam)
  }

  # Filter for requested chromosomes
  available_chr <- snps$chr %>%
    unique()
  if (!is.null(chromosomes)) {
    if (!any(chromosomes %in% available_chr)) {
      stop(paste0(
        'Chromosome names do not match SNP file:\n',
        'Requested: ',
        paste0(chromosomes, collapse = ', '),
        '\n',
        'Available: ',
        paste0(available_chr, collapse = ', '),
        '\n'
      ))
    } else {
      snps <- snps %>%
        dplyr::filter(chr %in% chromosomes)
    }
  }

  # Check that byPop is logical
  if (!is.logical(byPop)) {
    stop('byPop must be logical (TRUE/FALSE)')
  }

  # Calculate groupings

  # Print information about chromosomes
  if (verbose) {
    cat(paste0(
      'Analysing chromosomes:\n',
      paste0(unique(snps$chr), collapse = ', '),
      '\n'
    ))
  }
  # Collect (potentially filtered) ind details
  if (is.null(pop_pattern)) {
    ind <- ind_raw %>%
      dplyr::mutate(row_number = 1:dplyr::n())
  } else {
    ind <- ind_raw %>%
      dplyr::mutate(row_number = 1:dplyr::n()) %>%
      dplyr::filter(pop %in% pop_pattern)
  }

  # Throw error if ind has no rows.
  n_ind <- nrow(ind)
  if (n_ind <= 1) {
    if (is.null(pop_pattern)) {
      stop(paste0(
        n_ind,
        ' individuals in final, filtered ind file. Check ind file.'
      ))
    } else {
      stop(paste0(
        n_ind,
        ' individuals in final, filtered ind file. Check ind file, and check pop_pattern.'
      ))
    }
  }
  geno_list <- vector(mode = 'list', length = n_ind)

  if (verbose) {
    cat('Starting to read in genotype data.\n')
  }
  pb1 <- utils::txtProgressBar(
    min = 1,
    max = length(geno_list),
    initial = 1,
    style = 3
  )
  dt <- data.table::fread(genofile, col.names = "snp", colClasses = "character")
  for (i in 1:n_ind) {
    dt[, digit := stringr::str_sub(snp, ind$row_number[i], ind$row_number[i]), ]
    geno_list[[i]] <- (dt[[2]] %>% dplyr::na_if("9"))[snps$relative_pos]
    utils::setTxtProgressBar(pb1, i)
  }
  if (verbose) {
    cat('  Complete.\n\n')

    cat('Starting to compare genotypes and calculate PMR.\n')
  }

  if (!byPop) {
    pop_group <- rep('Merged', nrow(ind))
  } else {
    pop_group <- ind %>%
      dplyr::group_by(pop) %>%
      dplyr::mutate(N = dplyr::n()) %>%
      dplyr::ungroup() %>%
      dplyr::mutate(pop_group = ifelse(N < minMerge, 'Merged', pop)) %>%
      dplyr::pull(pop_group)
  }

  # Check if things work
  if (min(table(pop_group)) == 1) {
    stop('Even after possibly merging, smallest population group size is 1.')
  } else {
    POPS <- pop_group %>%
      base::unique() %>%
      base::sort()
    if (byPop) {
      if (sum(pop_group == 'Merged') > 0) {
        ind %>%
          dplyr::slice(which(pop_group == 'Merged')) %>%
          dplyr::pull(pop) %>%
          base::unique() %>%
          base::sort() %>%
          base::paste0(collapse = '/') %>%
          base::sprintf('Merged group made from: %s\n', .) %>%
          message()
      }
    }
  }

  for (k in 1:length(POPS)) {
    if (k == 1) {
      comp_list <- base::vector(mode = 'list', length = length(POPS))
    }

    currInds <- ind %>%
      dplyr::filter(pop_group == POPS[k])
    n_ind <- nrow(currInds)
    if (byPop) {
      sprintf(
        'Comparing within population: %s (%i comparisons)',
        POPS[k],
        choose(n_ind, 2)
      ) %>%
        message()
    }
    counter <- 1
    out_tibble <- tibble::tibble(
      pair = rep('x', choose(n_ind, 2)),
      nsnps = 0,
      mismatch = 0,
      pmr = 0
    )
    pb2 <- utils::txtProgressBar(
      min = 0,
      max = choose(n_ind, 2),
      initial = 1,
      style = 3
    )
    for (i in 1:(n_ind - 1)) {
      ind_i <- currInds$ind[i]
      vi <- geno_list[[i]]
      for (j in (i + 1):n_ind) {
        ind_j <- currInds$ind[j]
        vj <- geno_list[[j]]

        snps_filtered_prefiltered <- snps %>%
          dplyr::mutate(row_num = 1:dplyr::n()) %>%
          dplyr::filter(!is.na(vi), !is.na(vj))

        if (nrow(snps_filtered_prefiltered) > 0) {
          snps_filtered <- snps_filtered_prefiltered %>%
            dplyr::group_by(chr) %>%
            dplyr::mutate(keep = getfilter(site, filter_length)) %>%
            dplyr::ungroup() %>%
            dplyr::filter(keep) %>%
            dplyr::select(-keep)

          out_tibble$pair[counter] <- paste0(ind_i, ' - ', ind_j)
          out_tibble$nsnps[counter] <- (!is.na(vi[snps_filtered$row_num]) &
            !is.na(vj[snps_filtered$row_num])) %>%
            sum()
          out_tibble$mismatch[counter] <- (vi[snps_filtered$row_num] !=
            vj[snps_filtered$row_num]) %>%
            sum(na.rm = T)
          out_tibble$pmr[counter] <- out_tibble$mismatch[counter] /
            out_tibble$nsnps[counter]
        } else {
          out_tibble$pair[counter] <- paste0(ind_i, ' - ', ind_j)
          out_tibble$nsnps[counter] <- 0
          out_tibble$mismatch[counter] <- 0
          out_tibble$pmr[counter] <- NA
        }
        counter <- counter + 1
        utils::setTxtProgressBar(pb2, counter)
      }
    }
    comp_list[[k]] <- out_tibble
  }

  OUT_TIBBLE <- comp_list %>%
    dplyr::bind_rows()

  if (!is.null(outfile)) {
    if (verbose) {
      cat(sprintf('\n\nSaving processed data to %s.\n', outfile))
    }
    readr::write_delim(OUT_TIBBLE, file = outfile, delim = '\t')
  }
  if (verbose) {
    cat('\nComplete.\n\n')
  }

  OUT_TIBBLE %>%
    return()
}

test_that("processEigenstart errors", {
  # load files
  indfile <- system.file("extdata", "example.ind.txt", package = "BREADR")
  genofile <- system.file("extdata", "example.geno.txt", package = "BREADR")
  snpfile <- system.file("extdata", "example.snp.txt", package = "BREADR")
  # check input
  expect_error(
    processEigenstrat(indfile = "no-file.txt")
  )
  expect_error(
    processEigenstrat(indfile = indfile, snpfile = "no-file.txt")
  )
  expect_error(
    processEigenstrat(indfile = indfile,
                      snpfile = snpfile,
                      genofile = "no-file.txt")
  )
  expect_error(
    processEigenstrat(indfile = indfile,
                      snpfile = snpfile,
                      genofile = genofile,
                      filter_length = -1)
  )
  expect_error(
    processEigenstrat(indfile = indfile,
                      snpfile = snpfile,
                      genofile = genofile,
                      filter_deam = "NO")
  )
})

#' sim_geno
#'
#' Simulated geno file of eigenstrat format
#' 
#' @param n_ind number of individuals
#' @param n_snp number of SNPs
#' @param filename filename of export 
#'
#' @return NULL exports a file
#' @export
#'
#' @examples
#' \dontrun{
#' sim_geno(10, 5, "geno.txt")
#' }
sim_geno <- function(n_ind, n_snp, filename){
  N <- n_ind * n_snp
  M <- matrix(
    sample(c(0, 1, 2, 9), size = N, replace = TRUE), 
    nrow = n_snp, ncol = n_ind
  )
  MASS::write.matrix(M, file = filename, sep = "")
}
# pacman::p_load(conflicted, tidyverse, targets)
# sim_geno(10, 5, "geno.txt") |> print()

#' callRelatedness
#'
#'
#' A function that takes PMR observations, and (given a prior distribution for
#' degrees of relatedness) returns the posterior probabilities of all pairs of
#' individuals being (a) the same individual/twins, (b) first-degree related,
#' (c) second-degree related or (d) "unrelated" (third-degree or higher).
#' The highest posterior probability degree of relatedness is also returned as
#' a hard classification.
#' Options include setting the background relatedness (or using the sample
#' median), a minimum number of overlapping SNPs if one uses the sample median
#' for background relatedness, and a minimum number of overlapping SNPs
#' for including pairs in the analysis.
#'
#' @param pmr_tibble a tibble that is the output of the processEigenstrat
#' function.
#' @param class_prior the prior probabilities for same/twin, 1st-degree,
#' 2nd-degree, unrelated, respectively.
#' @param average_relatedness a single numeric value, or a vector of numeric
#' values, to use as the average background relatedness. If NULL, the sample
#' median is used.
#' @param median_co if average_relatedness is left NULL, then the minimum cutoff
#' for the number of overlapping snps to be included in the median calculation
#' is 500.
#' @param filter_n the minimum number of overlapping SNPs for which pairs are
#' removed from the entire analysis. If NULL, default is 1.
#'
#' @return   results_tibble: A tibble containing 13 columns:
#' - row: The row number
#' - pair: the pair of individuals that are compared.
#' - relationship: the highest posterior probability estimate of the degree of relatedness.
#' - pmr: the pairwise mismatch rate (mismatch/nsnps).
#' - sd: the estimated standard deviation of the pmr.
#' - mismatch: the number of sites which did not match for each pair.
#' - nsnps: the number of overlapping snps that were compared for each pair.
#' - ave_re;: the value for the background relatedness used for normalisation.
#' - Same_Twins: the posterior probability associated with a same individual/twins classification.
#' - First_Degree: the posterior probability associated with a first-degree classification.
#' - Second_Degree: the posterior probability associated with a second-degree classification.
#' - Unrelated: the posterior probability associated with an unrelated classification.
#' - BF: A strength of confidence in the Bayes Factor associated with the highest posterior probability classification compared to the 2nd highest. (No longer included)
#' @export
#'
#' @examples
#' callRelatedness(counts_example,
#'   class_prior=rep(0.25,4),
#'   average_relatedness=NULL,
#'   median_co=5e2,filter_n=1
#' )
callRelatedness <- function(
  pmr_tibble,
  class_prior = rep(0.25, 4),
  average_relatedness = NULL,
  median_co = 5e2,
  filter_n = 1
) {
  # Check pmr_tibble is of the form required:
  cols_check <- c('pair', 'nsnps', 'mismatch', 'pmr')
  if (!(all(cols_check %in% names(pmr_tibble)))) {
    stop(paste0(
      'Input tibble/data.frame must have 4 columns named: ',
      paste0(cols_check, collapse = ', '),
      '.'
    ))
  }

  # Check that the prior distribution makes sense:
  if (any(class_prior < 0) | any(class_prior > 1)) {
    stop(
      'Posterior probabilities for degrees of relatedness must be values between 0 and 1.'
    )
  } else if (abs(sum(class_prior) - 1) > 1e-3) {
    warning(
      'Posterior probabilities for degrees of relatedness must sum to 1. Normalising.'
    )
    class_prior <- class_prior / sum(class_prior)
  }

  # Check that the average relatedness makes sense:
  if (!is.null(average_relatedness)) {
    if (any(average_relatedness <= 0) | any(average_relatedness >= 1)) {
      stop('The average relatedness must be a value between 0 and 1.')
    }
  }

  # Check median estimator cut off:
  if ((median_co <= 0)) {
    stop(
      'The median cut off number of overlapping snps must be greater than 0.'
    )
  } else if (median_co >= max(pmr_tibble$nsnps, na.rm = T)) {
    stop(
      'The median cut off number of overlapping snps cannot be greater than the maximum number of overlapping SNPs that were observed!'
    )
  }

  # Check median estimator cut off:
  if ((filter_n <= 0)) {
    stop(
      'The cut off for the number of overlapping snps must be greater than 0.'
    )
  } else if (filter_n >= max(pmr_tibble$nsnps, na.rm = T)) {
    stop(
      'The cut off for the number of overlapping snps cannot be greater than the maximum number of overlapping SNPs that were observed!'
    )
  }

  # local variables
  class_vec <- c('Same_Twins', 'First_Degree', 'Second_Degree', 'Unrelated')

  # If no user-defined value for average_relatedness is given, use the median (above nsnps cut off)
  if (is.null(average_relatedness)) {
    M <- pmr_tibble %>%
      dplyr::filter(nsnps > median_co) %>%
      dplyr::pull(pmr) %>%
      stats::median(na.rm = T)
    filter_tibble <- pmr_tibble %>%
      dplyr::filter((nsnps >= filter_n)) %>%
      dplyr::rowwise() %>%
      dplyr::mutate(ave_rel = M)
  } else {
    M <- average_relatedness
    filter_tibble <- pmr_tibble %>%
      dplyr::mutate(ave_rel = M) %>%
      dplyr::filter((nsnps >= filter_n))
  }

  results_tibble <- filter_tibble %>%
    dplyr::ungroup() %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      Same_Twins = (stats::dbinom(mismatch, nsnps, 0.5 * ave_rel, log = T) +
        log(class_prior[1])),
      First_Degree = (stats::dbinom(
        mismatch,
        nsnps,
        (1 - 0.5^2) * ave_rel,
        log = T
      ) +
        log(class_prior[2])),
      Second_Degree = (stats::dbinom(
        mismatch,
        nsnps,
        (1 - 0.5^3) * ave_rel,
        log = T
      ) +
        log(class_prior[3])),
      Unrelated = weightedBinom(mismatch, nsnps, ave_rel, log = T) +
        log(class_prior[4])
    ) %>%
    dplyr::mutate(
      normConst = matrixStats::logSumExp(c(
        Same_Twins,
        First_Degree,
        Second_Degree,
        Unrelated
      )) %>%
        exp()
    ) %>%
    dplyr::mutate(
      relationship = class_vec[which.max(c(
        Same_Twins,
        First_Degree,
        Second_Degree,
        Unrelated
      ))]
    ) %>%
    dplyr::ungroup() %>%
    dplyr::mutate(
      Same_Twins = exp(Same_Twins) / normConst,
      First_Degree = exp(First_Degree) / normConst,
      Second_Degree = exp(Second_Degree) / normConst,
      Unrelated = exp(Unrelated) / normConst
    ) %>%
    dplyr::mutate(
      sd = sqrt(pmr * (1 - pmr) / nsnps),
      relationship = factor(
        relationship,
        levels = c('Same_Twins', 'First_Degree', 'Second_Degree', 'Unrelated')
      ),
      row = 1:dplyr::n()
    ) %>%
    dplyr::select(
      row,
      pair,
      relationship,
      pmr,
      sd,
      mismatch,
      nsnps,
      ave_rel,
      everything(),
      -normConst
    )
  return(results_tibble)
}

test_that("callRelatedness errors", {
  false_example1 <- counts_example %>% dplyr::rename(bob = pair)
  expect_error(callRelatedness(false_example1))
  expect_error(callRelatedness(counts_example, class_prior = -1))
  expect_error(callRelatedness(counts_example, class_prior = 1.1))
  expect_warning(callRelatedness(counts_example, class_prior = c(0.1, 0.1)))
  expect_error(callRelatedness(counts_example, average_relatedness = 1.1))
  expect_error(callRelatedness(counts_example, median_co = -1))
  expect_error(callRelatedness(counts_example, filter_n = -1))
  tmp <- callRelatedness(counts_example)
  expect_equal(nrow(tmp), 15)
  expect_equal(ncol(tmp), 12)
  expect_equal(tmp$pmr[1], 0.2042, tolerance = 0.001)
})

---
title: "Example BREADR Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Example BREADR Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(BREADR)
```

## Introduction

BREADR (Biological Relatedness from Ancient DNA in R) is a Bayesian method for identifying the degrees of relatedness between pairs of individuals, especially in cases of extremely low-coverage sequence data (as low as 0.04X). BREADR leverages the so-called pairwise mismatch rate (PMR), a measure of how much the genotypes of individuals do not match, to do this. BREADR assumes an underlying binomial model for the proportion of mismatched genotype calls, and calculates posterior probabilities for degrees of relatedness up to the second-degree (third-degree or higher are defined as "Unrelated"). BREADR works with the Eigenstrat data format as an initial input, allowing researchers to control for factors such as contamination before BREADR is used.

We have implemented BREADR in the R statistical programming language, and include methods for assigning degrees of relatedness (up to the second-degree), testing for whether separate groups of individuals can be merged into one analysis, for visualising both pairwise and overall relatedness, for sensitivity analyses for changes in the prior distributions, and for testing if degrees of relatedness up to the tenth-degree are consistent with the observed PMR. This basic example shows you how to analyse a real (anonymised) ancient DNA data set. 

## The theory behind BREADR

For a comprehensive description of the underlying model used by BREADR, please see the [manuscript](https://doi.org/10.21105/joss.07916). However, we give a brief overview of the concept here. Consider two individuals $i$ and $j$, with $N_{ij}$ independent sites in the genome for which genotype calls have been made. We count the number of genotypes which do not match, and denote this $X_{ij}$. We assume that $X_{ij}$ will follow a binomial distribution $X_{ij} \sim B(N_{ij},p_{ij})$, that $p_{ij}$ is defined by how closely related the two individuals are.

It is known that if the expected PMR of two unrelated individuals is $\bar{p}$, then the expected PMR for individuals how are related in the $k^{\text{th}}$ degree will be

$$
  p_{k} = \bar{p}\left( 1 - \dfrac{1}{2^{k+1}} \right).
$$

Since the probability mass function for a binomial distribution is well known, we can calculate the probability of observing the number of mismatched genotypes for $k=0$ (same/twins), $k=1$ (first-degree), $k=2$ (second-degree), and then a Poisson-weighted sum of the remaining values of $k$ to capture the third-degree or less related probability.

When we have the probability of observing the number of mismatches we see, given a value of $k$, denoted $L(X_{ij} | K=k,N_{ij})$, and assuming a (user-defined) prior probability of relatedness, we then calculate a posterior probability of the degree of relatedness being $k$, _given_ the observed PMR.

## Processing your data

An analysis would normally start by defining paths to all three Eigenstrat files, i.e.,

```{r}
ind_path <- "path/to/eigenstrat/indfile"
snp_path <- "path/to/eigenstrat/snpfile"
geno_path <- "path/to/eigenstrat/genofile"
```

and we would then preprocess this data using the processEigenstrat() function,

```{r,eval=FALSE}
counts_example <- processEigenstrat(
  indfile = ind_path,
  snpfile = snp_path,
  genofile = geno_path
)
```

Since this step is the most computationally expensive, we include an option to automatically save the output as a TSV upon completion of the preprocessing, i.e, to avoid repeating the preprocessing

```{r,eval=FALSE}
counts_example <- processEigenstrat(
  indfile = ind_path,
  snpfile = snp_path,
  genofile = geno_path,
  outfile = "path_to_save_tsv"
)
```

Also included are options to change the minimum distance between overlapping sites (filter_length) from the default $1\times 10^5,$ an option to only include individuals (pop_pattern) from certain populations (as defined in the ind file) and an option to remove C->T and G->A SNPs to avoid the effects of deamination.

## Processing only pairs within groups to save computational effort

A new feature is the ability to process only pairs _within_ the same populations (as per the populations defined in column three of the Eigenstrat ind file). This can be useful when populations should not be considered together as, for example, they may never have been able to be closely related due to vast distances in time or geography, and/or might have a different baseline PMR. When running the processEigenstrat() function, by setting the parameter *byPop=TRUE*, only individuals in the same population will have a PMR calculated. In the example data, individuals Ind1-Ind4 are in "Pop1" and individuals Ind5-Ind6 are in "Pop2". This means that while each genotype of each individual will be read into BREADR, only the four individuals in Pop1 are compared to each other ($4\times 3/2 = 6$ pairs) and only the two individuals in Pop2 are compared to each other ($2\times 1/2 =1$ pairs), totaling 7 PMR calculations. This is less than the $6\times 5/2 = 15$ pairs that would be calculated by default, more than halving the number of calculations. This saving in computational effort will be greater for large sample sizes. We also include a related parameter minMerge which defines the smallest possible "group" (default equals 2). This is to control the smallest group sizes as estimating a baseline PMR can be difficult for small groups (such as in Pop2 in the example data), and any groups with a sample size smaller than this parameter are merged into one group called "Merged" (hence you should avoid this as a population name in your Eigenstrat ind file when using this parameter).

Note that since the Eigenstrat files are too large to include, we provide a pre-processed data set called [counts_example](https://github.com/jonotuke/BREADR/blob/master/data/counts_example.rda) which is the product of running processEigenstrat() on real data (although the raw Eigenstrat files can be found on [the Github page](https://github.com/jonotuke/BREADR/tree/master/data-raw)). 

We now load the counts_example tibble. 

```{r example, eval = FALSE}
library(BREADR)
counts_example
```

```{r, echo = FALSE}
counts_example
```

## Calling degrees of relatedness from counts

We can estimate the degrees of relatedness from the raw counts using the callRelatedness() function. Users have the option to change the prior probability for each degree of relatedness (class_prior) from the default uniform prior, to define the expected PMR (average_relatedness) for a pair of unrelated individuals from the default of the sample median, and to set the minimum number of overlapping SNPs required for a pair of individuals (filter_n) to be included in the analysis from the default of 1. If the user decides to use the sample median as an estimate for the expected PMR for a pair of unrelated individuals, the minimum number of overlapping SNPs for a pair (median_co) can be changed from the default of 500.

```{r, eval = FALSE}
relatedness_example <- callRelatedness(counts_example)
relatedness_example
```

```{r, echo = FALSE}
relatedness_example <- callRelatedness(counts_example)
relatedness_example
```

For each pair of individuals, a posterior probability for same/twins, first-degree, second-degree and unrelated will be calculated and reported in the final four columns. However, the relationship that is reported is simply the relationship that had the _highest posterior probability_, no matter how high the value of the second highest posterior probability is. In the section titled "Visualising the relatedness information for two individuals" we will see why it can be important to look at these values for "borderline" assignments, and how this can be easily done using BREADR.

## Assessing if groups can be merged

We now also include a diagnostic plot to consider whether populations _can be merged_. We may wish to merge populations (we use population instead of group to match the Eigenstrat nomenclature for the third column of the IND file) if we have a small number of individuals, and wish to increase the _pairwise_ sample size.

The plotDOUGH() function takes as input your processEigenstrat() output (the raw counts) and produces side-by-side box plots/violin plots, comparing the PMR values _per population_, with associated non-parametric tests for whether the median PMR (which we use as a baseline level of relatedness by default in BREADR) differs between populations. When employing this function the user can set parameters for the font size on the x-axis (fntsize, default equals 7), the minimum number of overlapping SNPs for a pair to be included in the tests and plot (nsnps_cutoff, default is 500) and removeBetween. removeBetween forces the test to ignore the between-population PMR values, which may be useful.

In cases where there are more than two populations, BREADR first performs a Kruskal-Wallis test to see if _any_ population medians significantly differ (p-value in top-left), and then performs pairwise posthoc Dunn's tests to identify _which_ population medians are significantly different, with p-value adjustments performed using the Holm-Bonferroni method. Populations with significantly differently medians are then identified in the plot as bars above the population (with p-values). If there are only two population (and removeBetween is set to TRUE) then a Mann-Whitney U-test is performed and no bars will be shown even if the groups are significantly different, so the p-value in the title must be used to assess significance. If two populations are not significantly different, then they can, in theory, be merged.

We run plotDOUGH() on our example in the following way:

```{r, fig.dim=c(7.2,5), warning=FALSE, message=FALSE, fig.cap = '*__Figure 1__: plotDOUGH results for the example data set, with the Between Groups population kept.*'}
indfile1 <- fs::path_package(package = "BREADR", "extdata/example.ind.txt")
plotDOUGH(
  counts_example,
  indfile = indfile1,
  nsnps_cutoff = 500
)
```

In this example (Figure 1), the Kruskal-Wallis test returns a p-value of 0.57 (>0.05), indicating that there is no evidence that any population median is different, and hence our two populations can be merged. However, these non-parametric tests can lack statistical power for very small sample sizes, as might be the case here. However, as an example of when there is a difference, we modified the example data such that Pop1 and Pop2 cannot be merged. (Note that the "Between" groups PMR values are not significantly different from Pop1 or Pop2, but this is only because they are the average of the two groups, and in some sense should be ignored here).

```{r, echo=FALSE, fig.dim=c(7.2,5), warning=FALSE, message=FALSE, fig.cap = '*__Figure 2__: plotDOUGH results for the example data set where the PMR values have been artifically modified to yield different baseline (median) PMR values.*'}
indfile2 <- fs::path_package(package = "BREADR", "extdata/example2.ind.txt")
counts_example %>%
  dplyr::mutate(
    mismatch = dplyr::case_when(
      stringr::str_count(pair, 'Ind[123]') == 2 ~ floor(0.22 * nsnps),
      stringr::str_count(pair, 'Ind[456]') == 2 ~ floor(0.16 * nsnps),
      T ~ floor(0.19 * nsnps)
    ) +
      sample(10:20, dplyr::n(), replace = TRUE),
    pmr = mismatch / nsnps
  ) %>%
  plotDOUGH(
    indfile = indfile2,
    nsnps_cutoff = 500
  )
```



## Visualising overall relatedness patterns

An overall picture of the relatedness for these individuals can be created (Figure 3), which displays the highest posterior probability degree of relatedness for each pair of individuals, indicated by the colour and shape of the points with 95% confidence intervals. The dashed horizontal lines indicated the expected values for each degree of relatedness.

```{r, warning=FALSE, message=FALSE, fig.dim=c(7.2,5), fig.cap = '*__Figure 3__: Pairwise diagnostic plot for relatedness call for Ind1 and Ind 2 called using the number option for row. (A) the distribution of PMR values given the number of overlapping SNPs (n=1518) for each degree of relatedness (filled colour), and the observed PMR with 95% confidence intervals. (B) relative posterior probabilities for all four possible assignments of degrees of relatedness. Note we added panel labels A and B using the labels parameter.*'}
plotLOAF(relatedness_example)
```

Users can choose to remove individuals with less than a user-defined number of overlapping SNPs (nsnps_cutoff). In large data sets, it is expected that the vast majority of individuals will be unrelated, and so the user may wish to only plot the more closely related individuals. To this end, the user may choose to plot only the first $N$ individuals (using the  input parameter N), which have been sorted by PMR value, in ascending order.

## Visualising the relatedness information for two individuals

A researcher may be interested in exploring the relatedness between two specific individuals in the analysis. These individuals may be especially important to the study or may have a PMR that is "uncertain" (i.e., falls between two of the expected values for degrees of relatedness). 

A plot for the assignment of the "Unrelated" degree of relatedness to individuals Ind1 and Ind2 can be produced which shows diagnostic information about the estimated genetic relatedness between a single pair of individuals (Figure 4). In the left panel, the distribution of PMR values for each degree of relatedness, given the number of overlapping SNPs, with the observed PMR (and 95% confidence interval) displayed below this. In the right panel are the normalised posterior probabilities for each of the degrees of relatedness, indicating the certainty with which the degree of relatedness with the highest posterior probability was chosen.

```{r, fig.dim=c(7.2,5), warning=FALSE, fig.cap = '*__Figure 4__: Pairwise diagnostic plot for relatedness call for Ind1 and Ind 2 called using the text option for row. (A) the distribution of PMR values given the number of overlapping SNPs (n=1518) for each degree of relatedness (filled colour), and the observed PMR with 95% confidence intervals. (B) relative posterior probabilities for all four possible assignments of degrees of relatedness. Note we added panel labels A and B using the labels parameter.*'}
plotSLICE(relatedness_example, row = 1, labels = c('A', 'B'))
```

Alternatively we can create an identical plot, but this time choose the row using the full pair name instead of the row number (Figure 5).

```{r, fig.dim=c(7.2,5), warning=FALSE, fig.cap = '*__Figure 5__: Pairwise diagnostic plot for relatedness call for Ind1 and Ind 2 called using the text option for row. (A) the distribution of PMR values given the number of overlapping SNPs (n=1518) for each degree of relatedness (filled colour), and the observed PMR with 95% confidence intervals. (B) relative posterior probabilities for all four possible assignments of degrees of relatedness. Note we added panel labels A and B using the labels parameter.*'}
plotSLICE(relatedness_example, row = "Ind1 - Ind2", labels = c('A', 'B'))
```

From these we make the following observations. First, in panel A we see that the observed PMR falls between the distributions for second-degree and unrelated. Second, in panel B we see that the posterior probabilities for both assignments are relatively similar (even if the highest posterior assignment was reported as unrelated). Unfortunately, this is the limit of the resolution of BREADR, as we cannot assign a third-degree relationship.

## Binomial tests for degrees of relatedness

Since we assume that the number of mismatches follows a binomial distribution, and since we know the expected rate of mismatch for any value of $k$, we can test if the observed PMR is consistent with a $k^{\text{th}}$-degree of relatedness using a binomial hypothesis test. This is done using the test_degree function where again the row of interest must be given, and the degree ($k$) is specified.

For example, we can test whether the observed PMR between Ind1 and Ind2 is consistent with a 3rd-degree genetic relationship, or not, but any degree of relatedness from 0 to 10 (including non-integer values) can be investigated. This function returns the p-value from the associated binomial test, and by setting verbose to TRUE, all diagnostic output from the test can be displayed.

```{r}
test_degree(relatedness_example, row = 1, degree = 3, verbose = TRUE)
```

Here we see that Ind1 and Ind2 yield a PMR consistent with a third-degree relationship (at the 5% significance level, as the p-value is greater than 0.05). If the verbose parameter is TRUE, then more than the p-value is returned. Specifically, we see (in order) the null hypothesis, the PMR we would expected for a third-degree relationship, the PMR we actually observed, the estimated degree of relatedness (calculated by inverting the equation for $p_k$), the p-value and the decision for the test (at the 5% significance level).

## Sensitivity of the results to the prior distribution

By default, BREADR sets the prior distribution for the four possible relatedness assignments as equally likely, or a uniform prior of $P(K=k) = \frac{1}{4}$. This is likely not reasonable in a random sample from a true population. However, an uninformative prior makes the least assumptions, and on a practical level, the PMR associated with same/twin or first-degree can often be overwhelmingly obvious for a sufficient number of overlapping sites. Further, some studies will contain random samples from populations, and hence twins will be very rare. Others samples may be from collective burials (for example), in which case elements from the same individuals may be extremely likely. Hence, we leave it to the user to decide on a sensible prior distribution, but concede that this can be difficult. 

To address this we include a function called priorSensitivity() to allow researchers to easily explore how sensitive the results of callRelatedness() are to the choice of the prior distribution. The function forms a grid of different prior probabilities to test, and then for each degree of relatedness, at each prior value, calculates the proportion of times each degree of relatedness would have been the "assigned" relationship (when all other priors are varied). Consider the example for Ind1 and Ind2 (row one) in the example data (Figure 6).

```{r, fig.dim=c(7.2,5), warning=FALSE, message=FALSE, fig.cap = '*__Figure 6__: sensitivity analysis for the relationship between Ind1 and Ind2. The prior probabilities for all degrees of relatedness are tested at values between 0.05 and 0.5. Bars indicate the proportion of times a degree of relatedness was the maximum posterior relationship. Here we see that higher prior probabilities for Second-Degree and Unrelated yield highest posterior probability assignments for those degrees of relatedness, respectively. Hence, we must consider these prior probabilities carefully when assigning this relationship.*'}
BREADR::priorSensitivity(
  in_BREADR = relatedness_example,
  row = "Ind1 - Ind2",
  degrees = c(3, 4),
  grid_space = 0.05,
  maxPrior = 0.5
)
```

This output can be difficult to interpret at first, and so we explain it here in detail. From Figure 5 we can see two panels: 2nd-Degree and Unrelated, both with a y-axis from 0% to 100%, and an x-axis that goes from 0 to the (default) maximum tested prior probability of 0.5. For the prior probability value along the x-axis, we see a bar. The way to interpret the first bar (at $x=0.05$) is: when the prior probability of _Unrelated_ is fixed at 0.05, and across all other values that the other three prior values can be (so that all four prior probabilities sum to one), we see that the relationship between Ind1 and Ind2 is returned as "Unrelated" approximately 4% of time, "2nd-Degree" approximately 96% of the time, and never as "Same/Twins" or "1st-Degree", as we never see the colours associated with them. We can see that the next bar ($x=0.1$) sees "2nd-degree" assigned most often, but less often than before. And this trend continues. A sensible interpretation of this is that as the prior probability of "Unrelated" increases, we see that the relationship between Ind1 and Ind2 is more likely to be called "Unrelated", which makes sense. We also see that we would need a prior probability of unrelated of at least 0.25 to call an unrelated relationship. 

Since Same/Twins and 1st-degree is never called, the top panel for 2nd-Degree effectively shows the inverse of the bottom panel. Unfortunately this sensitivity analysis shows us that the outcome of BREADR is strongly affected by the prior distribution here, which makes sense since it appears as if the relationship is potentially third-degree.

To give an example of when a degree of relatedness is not affected by the choice of prior probabilities, we produce a sensitivity analysis for Ind3 and Ind4 (Figure 7), who are unambiguously Same/Twins with posterior probability one (to machine precision). We also decrease the grid spacing (grid_space=0.025) to have a finer-scale analysis (at the cost of computational time). Here we can see that no matter how we set the prior distribution for Same/Twins, even as low as 0.025, we still always assign Same/Twins. Hence, this assignment is free of any affect from the prior distribution.

```{r, fig.dim=c(7.2,3.5), warning=FALSE, message=FALSE,fig.cap = '*__Figure 7__: sensitivity analysis for the relationship between Ind3 and Ind4. The prior probabilities for all degrees of relatedness are tested at values between 0.05 and 0.5. Bars indicate the proportion of times a degree of relatedness was the maximum posterior relationship, and the dashed black line indicates 50%. This clearly shows that no matter the prior probability values for any of the degrees of relatedness, the highest posterior probability assignment is always Same/Twins.*'}
BREADR::priorSensitivity(
  in_BREADR = relatedness_example,
  row = "Ind3 - Ind4",
  degrees = c(1),
  grid_space = 0.025,
  maxPrior = 0.5
)
```

The priorSensitivity() function can also be adjusted with input parameters. First, we can focus on any combination of the degrees of relatedness, and a vector containing any combination of 1, 2, 3 or 4 will specify Same/Twin, 1st-Degree, 2nd-Degree or Unrelated to be explored, respectively. For example, to look at just the affect on calling Same/Twins, set degree=1, or for 2nd-Degree _and_ Unrelated, degree=c(3,4). Second, the maximum value for the prior probability (the upper bound of the x-axis) can be changed using the maxPrior parameter (default is 0.5). Finally, grid_space defines the space between tested prior probability values (the width of the bars on the x-axis). The smaller this number, the more bars are calculated, but this comes at the expense of computational time (default is 0.05).

## Acknowledgements

The authors wish to thank Beatriz Amorim for help testing the functionality of BREADR, and Prof. Nigel Bean and Dr. Vincent Braunack-Mayer for enlightening and instructive conversations.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/callRelatedness.R
\name{callRelatedness}
\alias{callRelatedness}
\title{callRelatedness}
\usage{
callRelatedness(
  pmr_tibble,
  class_prior = rep(0.25, 4),
  average_relatedness = NULL,
  median_co = 500,
  filter_n = 1
)
}
\arguments{
\item{pmr_tibble}{a tibble that is the output of the processEigenstrat
function.}

\item{class_prior}{the prior probabilities for same/twin, 1st-degree,
2nd-degree, unrelated, respectively.}

\item{average_relatedness}{a single numeric value, or a vector of numeric
values, to use as the average background relatedness. If NULL, the sample
median is used.}

\item{median_co}{if average_relatedness is left NULL, then the minimum cutoff
for the number of overlapping snps to be included in the median calculation
is 500.}

\item{filter_n}{the minimum number of overlapping SNPs for which pairs are
removed from the entire analysis. If NULL, default is 1.}
}
\value{
results_tibble: A tibble containing 13 columns:
\itemize{
\item row: The row number
\item pair: the pair of individuals that are compared.
\item relationship: the highest posterior probability estimate of the degree of relatedness.
\item pmr: the pairwise mismatch rate (mismatch/nsnps).
\item sd: the estimated standard deviation of the pmr.
\item mismatch: the number of sites which did not match for each pair.
\item nsnps: the number of overlapping snps that were compared for each pair.
\item ave_re;: the value for the background relatedness used for normalisation.
\item Same_Twins: the posterior probability associated with a same individual/twins classification.
\item First_Degree: the posterior probability associated with a first-degree classification.
\item Second_Degree: the posterior probability associated with a second-degree classification.
\item Unrelated: the posterior probability associated with an unrelated classification.
\item BF: A strength of confidence in the Bayes Factor associated with the highest posterior probability classification compared to the 2nd highest. (No longer included)
}
}
\description{
A function that takes PMR observations, and (given a prior distribution for
degrees of relatedness) returns the posterior probabilities of all pairs of
individuals being (a) the same individual/twins, (b) first-degree related,
(c) second-degree related or (d) "unrelated" (third-degree or higher).
The highest posterior probability degree of relatedness is also returned as
a hard classification.
Options include setting the background relatedness (or using the sample
median), a minimum number of overlapping SNPs if one uses the sample median
for background relatedness, and a minimum number of overlapping SNPs
for including pairs in the analysis.
}
\examples{
callRelatedness(counts_example,
  class_prior=rep(0.25,4),
  average_relatedness=NULL,
  median_co=5e2,filter_n=1
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveSLICES.R
\name{saveSLICES}
\alias{saveSLICES}
\title{saveSLICES}
\usage{
saveSLICES(
  in_tibble,
  outFolder = NULL,
  width = 297,
  height = 210,
  units = "mm",
  verbose = TRUE
)
}
\arguments{
\item{in_tibble}{a tibble that is the output of the callRelatedness() function.}

\item{outFolder}{the folder into which all diagnostic plots will be saved}

\item{width}{the width of the output PDFs.}

\item{height}{the height of the output PDFs.}

\item{units}{the units for the height and width of the output PDFs.}

\item{verbose}{Controls the printing of progress to console.}
}
\value{
nothing
}
\description{
Plots all pairwise diagnostic plots (in a tibble as output by callRelatedness),
as produced by plotSLICE, to a folder.
Options include the width and height of the output files, and the units in
which these dimensions are measured.
}
\examples{
\donttest{
saveSLICES(relatedness_example[1:3, ], outFolder=tempdir())
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relatedness_example.R
\docType{data}
\name{relatedness_example}
\alias{relatedness_example}
\title{relatedness_example}
\format{
\subsection{\code{relatedness_example}}{

A data frame with 15 rows and 13 columns:
\describe{
\item{row}{The row number}
\item{pair}{the pair of individuals that are compared.}
\item{relationship}{the highest posterior probability estimate of the degree of relatedness.}
\item{pmr}{the pairwise mismatch rate (mismatch/nsnps).}
\item{sd}{the estimated standard deviation of the pmr.}
\item{mismatch}{the number of sites which did not match for each pair.}
\item{nsnps}{the number of overlapping snps that were compared for each pair.}
\item{ave_re}{ the value for the background relatedness used for normalisation.}
\item{Same_Twins}{the posterior probability associated with a same individual/twins classification.}
\item{First_Degree}{the posterior probability associated with a first-degree classification.}
\item{Second_Degree}{the posterior probability associated with a second-degree classification.}
\item{Unrelated}{the posterior probability associated with an unrelated classification.}
\item{BF}{A strength of confidence in the Bayes Factor associated with the highest posterior probability classification compared to the 2nd highest.}
}
}
}
\usage{
relatedness_example
}
\description{
this is an example of the tibble made by callRelatedness()
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priorSensitivity.R
\name{priorSensitivity}
\alias{priorSensitivity}
\title{priorSensitivity}
\usage{
priorSensitivity(
  in_BREADR,
  row,
  degrees = NULL,
  grid_space = 0.05,
  maxPrior = 0.5
)
}
\arguments{
\item{in_BREADR}{A tibble containing the output of callRelatedness().}

\item{row}{Either the row number or pair name for which the sensitivity analysis should be run.}

\item{degrees}{A vector of integers identifying which degrees of relatedness degrees to plot. Same/Twins (1), First-Degree (2), Second-Degree (3) and Unrelated (4).}

\item{grid_space}{The space between prior probability values that are tested (smaller values mean a finer grid size).}

\item{maxPrior}{Maximum value a prior probability can take (the right-hand limit of the x-axis).}
}
\value{
plot
}
\description{
priorSensitivity
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotDOUGH.R
\name{plotDOUGH}
\alias{plotDOUGH}
\title{plotDOUGH}
\usage{
plotDOUGH(
  in_tibble,
  indfile,
  nsnps_cutoff = NULL,
  fntsize = 7,
  verbose = TRUE,
  removeBetween = FALSE
)
}
\arguments{
\item{in_tibble}{A tibble containing the output from processEigenstrat().}

\item{indfile}{Path to an ind file with columns for the individuals IDs, genetic sex (not used) and population.}

\item{nsnps_cutoff}{Minimum number of overlapping sites for a pair to be included.}

\item{fntsize}{Font size for x axis tick labels.}

\item{verbose}{Controls the printing of progress to console.}

\item{removeBetween}{TRUE/FALSE for whether to include the “between population” PMR values as their own group (FALSE), or to exclude them (TRUE).}
}
\value{
plots
}
\description{
plotDOUGH
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_geno.R
\name{sim_geno}
\alias{sim_geno}
\title{sim_geno}
\usage{
sim_geno(n_ind, n_snp, filename)
}
\arguments{
\item{n_ind}{number of individuals}

\item{n_snp}{number of SNPs}

\item{filename}{filename of export}
}
\value{
NULL exports a file
}
\description{
Simulated geno file of eigenstrat format
}
\examples{
\dontrun{
sim_geno(10, 5, "geno.txt")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_snp.R
\name{split_line}
\alias{split_line}
\title{split line}
\usage{
split_line(x)
}
\arguments{
\item{x}{line from SNP file}
}
\value{
tibble with 6 columns.
}
\description{
takes a line for a SNP file and splits into parts.
}
\examples{
split_line("1_14.570829090394763     1        0.000000              14 A X")
split_line("rs3094315	1	0.0	752566	G	A")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotLOAF.R
\name{plotLOAF}
\alias{plotLOAF}
\title{plotLOAF}
\usage{
plotLOAF(in_tibble, nsnps_cutoff = NULL, N = NULL, fntsize = 7, verbose = TRUE)
}
\arguments{
\item{in_tibble}{a tibble that is the output of the callRelatedness() function.}

\item{nsnps_cutoff}{the minimum number of overlapping SNPs for which
pairs are removed from the plot. If NULL, default is 500.}

\item{N}{the number of (sorted by increasing PMR) pairs to plot.
Avoids plotting all pairs (many of which are unrelated).}

\item{fntsize}{the fontsize for the x-axis names.}

\item{verbose}{if TRUE, then information about the plotting process is sent
to the console}
}
\value{
a ggplot object
}
\description{
Plots all (sorted by increasing value) observed PMR values with
maximum posterior probability classifications represented by colour
and shape.
Options include a cut off for the minimum number of overlapping SNPs,
the max number of pairs to plot and x-axis font size.
}
\examples{
relatedness_example
plotLOAF(relatedness_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_snp.R
\name{read_snp}
\alias{read_snp}
\title{read_snp}
\usage{
read_snp(filename)
}
\arguments{
\item{filename}{a SNP text file.}
}
\value{
tibble with column headings: snp (CHR), chr (DBL),
pos (DBL), site (DBL), anc (CHR), and der (CHR).
}
\description{
read_snp
}
\examples{
std_snpfile <- system.file("extdata", "example.snp.txt", package = "BREADR")
broken_snpfile <- system.file("extdata", "broken.snp.txt", package = "BREADR")
read_snp(std_snpfile)
read_snp(broken_snpfile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/counts_example.R
\docType{data}
\name{counts_example}
\alias{counts_example}
\title{counts_example}
\format{
\subsection{\code{counts_example}}{

A data frame with 15 rows and 4 columns:
\describe{
\item{pair}{the pair of individuals that are compared}
\item{nsnps}{the number of overlapping snps that were compared for each pair.}
\item{mismatch}{the number of sites which did not match for each pair.}
\item{pmr}{the pairwise mismatch rate (mismatch/nsnps).}
}
}
}
\usage{
counts_example
}
\description{
this is an example of the tibble made by processEigenstrat().
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_column.R
\name{get_column_new}
\alias{get_column_new}
\title{get column}
\usage{
get_column_new(genofile, col = 1)
}
\arguments{
\item{genofile}{genofile}

\item{col}{column to return}
}
\value{
column of numbers
}
\description{
get column
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSLICE.R
\name{plotSLICE}
\alias{plotSLICE}
\title{plotSLICE}
\usage{
plotSLICE(
  in_tibble,
  row,
  title = NULL,
  class_prior = rep(1/4, 4),
  showPlot = TRUE,
  which_plot = 0,
  labels = NULL
)
}
\arguments{
\item{in_tibble}{a tibble that is the output of the callRelatedness() function.}

\item{row}{either the row number or pair name for which the
posterior distribution is to be plotted.}

\item{title}{an optional title for the plot. If NULL, the pair
from the user-defined row is used.}

\item{class_prior}{the prior probabilities for same/twin, 1st-degree,
2nd-degree, unrelated, respectively.}

\item{showPlot}{If TRUE, display plot. If FALSE, just pass plot as a variable.}

\item{which_plot}{if 1, returns just the plot of the posterior distributions,
if 2 returns just the normalised posterior values. Anything else returns both
plots.}

\item{labels}{a length two character vector of labels for plots. Default is
no labels.}
}
\value{
a two-panel diagnostic ggplot object
}
\description{
A function for plotting the diagnostic information when classifying a
specific pair (defined by the row number or pair name) of individuals.
Output includes the PDFs for each degree of relatedness (given the number of
overlapping SNPs) in panel A, and the normalised posterior probabilities
for each possible degree of relatedness.
}
\examples{
plotSLICE(relatedness_example, row=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_ind.R
\name{read_ind}
\alias{read_ind}
\title{read_ind}
\usage{
read_ind(filename)
}
\arguments{
\item{filename}{a IND text file.}
}
\value{
tibble with column headings: ind (CHR), sex (CHR), pop (CHR)
}
\description{
read_ind
}
\examples{
ind_snpfile <- system.file("extdata", "example.ind.txt", package = "BREADR")
read_ind(ind_snpfile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/processEigenstrat.R
\name{processEigenstrat}
\alias{processEigenstrat}
\title{process Eigenstrat data - alternative version}
\usage{
processEigenstrat(
  indfile,
  genofile,
  snpfile,
  filter_length = NULL,
  pop_pattern = NULL,
  filter_deam = FALSE,
  outfile = NULL,
  chromosomes = NULL,
  verbose = TRUE,
  byPop = FALSE,
  minMerge = 2
)
}
\arguments{
\item{indfile}{path to eigenstrat ind file}

\item{genofile}{path to eigenstrat geno file.}

\item{snpfile}{path to eigenstrat snp file.}

\item{filter_length}{the minimum distance between sites to be compared (to reduce the effect of LD).}

\item{pop_pattern}{a character vector of population names to filter the ind file if only some populations are to compared.}

\item{filter_deam}{a TRUE/FALSE for if C->T and G->A sites should be ignored.}

\item{outfile}{(OPTIONAL) a path and filename to which we can save the output of the function as a TSV,
if NULL, no back up saved. If no outfile, then a tibble is returned.}

\item{chromosomes}{the chromosome to filter the data on.}

\item{verbose}{controls printing of messages to console}

\item{byPop}{Boolean variable identifying if all pairs of individuals should be compared (FALSE), or just pairs of individuals in the same populations (TRUE)}

\item{minMerge}{(integer) the minimum group size (smaller groups are merged).}
}
\value{
out_tibble: A tibble containing four columns:
}
\description{
A function that takes paths to an eigenstrat trio (ind, snp and geno file) and
returns the pairwise mismatch rate for all pairs on a thinned set of SNPs.
Options include choosing thinning parameter, subsetting by population names,
and filtering out SNPs for which deamination is possible.
}
\examples{
# Use internal files to the package as an example
indfile <- system.file("extdata", "example.ind.txt", package="BREADR")
genofile <- system.file("extdata", "example.geno.txt", package="BREADR")
snpfile <- system.file("extdata", "example.snp.txt", package="BREADR")
processEigenstrat(
indfile, genofile, snpfile,
filter_length=1e5,
pop_pattern=NULL,
filter_deam=FALSE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_degree.R
\name{test_degree}
\alias{test_degree}
\title{test_degree}
\usage{
test_degree(in_tibble, row, degree, verbose = TRUE)
}
\arguments{
\item{in_tibble}{a tibble that is the output of the callRelatedness() function.}

\item{row}{either the row number or pair name for which the posterior distribution is to be plotted.}

\item{degree}{the degree of relatedness to be tested.}

\item{verbose}{a logical (boolean) for whether all test output should be printed to screen.}
}
\value{
the associated p-value for the test
}
\description{
Test if a degree of relatedness is consistent with an observed PMR
}
\examples{
test_degree(relatedness_example, 1, 1)
}
